<?php

/**
 * Creates the cart post type for use with EDD and WooCommerce and handles
 * saving / loading / deleting carts.
 *
 * @since 1.7.0
 */
class WPF_Abandoned_Cart_Carts {

	/**
	 * Get things started.
	 *
	 * @since 1.7.0
	 */

	public function __construct() {

		add_action( 'init', array( $this, 'register_post_type' ) );
		add_action( 'add_meta_boxes', array( $this, 'add_meta_box' ) );
		add_action( 'wp_fusion_process_abandoned_carts', array( $this, 'process_abandoned_carts' ) );
		add_action( 'wp_fusion_abandoned_cart_cleanup', array( $this, 'cleanup' ) );

		if ( ! wp_next_scheduled( 'wp_fusion_abandoned_cart_cleanup' ) ) {
			wp_schedule_event( time(), 'daily', 'wp_fusion_abandoned_cart_cleanup' );
		}

		if ( wpf_get_option( 'abandoned_cart_delay' ) ) {
			add_filter( 'cron_schedules', array( $this, 'process_abandoned_carts_cron_schedule' ) );
		}

		if ( wpf_get_option( 'abandoned_cart_delay' ) && ! wp_next_scheduled( 'wp_fusion_process_abandoned_carts' ) ) {

			// Schedule the cart checking cron job.
			wp_schedule_event( time(), 'wpf_fifteen_minutes', 'wp_fusion_process_abandoned_carts' );

		} elseif ( ! wpf_get_option( 'abandoned_cart_delay' ) && wp_next_scheduled( 'wp_fusion_process_abandoned_carts' ) ) {

			// Clear the cart checking cron job.
			wp_clear_scheduled_hook( 'wp_fusion_process_abandoned_carts' );
		}
	}


	/**
	 * Register the abandoned cart post type.
	 *
	 * @since 1.7.0
	 */
	public function register_post_type() {

		$args = array(
			'label'               => __( 'Carts', 'wp-fusion-abandoned-cart' ),
			'description'         => __( 'WP Fusion Abandoned Carts', 'wp-fusion-abandoned-cart' ),
			'supports'            => array( 'title', 'custom-fields' ),
			'hierarchical'        => false,
			'public'              => false,
			'show_ui'             => true,
			'show_in_menu'        => false,
			'show_in_admin_bar'   => false,
			'show_in_nav_menus'   => false,
			'can_export'          => false,
			'has_archive'         => false,
			'exclude_from_search' => true,
			'publicly_queryable'  => false,
			'capability_type'     => 'page',
			'show_in_rest'        => false,
			'rewrite'             => false,
		);

		register_post_type( 'wpf_cart', $args );
	}

	/**
	 * Adds the cart contents meta box to the cart post type.
	 *
	 * @since 2.0.0
	 */
	public function add_meta_box() {

		add_meta_box(
			'wpf_cart_contents_meta_box',       // Unique ID.
			'Cart Contents',                    // Box title,
			array( $this, 'display_meta_box' ), // Content callback, must be of type callable.
			'wpf_cart',                         // Post type.
			'normal',                           // Context (normal, side, advanced).
			'high'                              // Priority (default, core, high, low).
		);
	}

	/**
	 * Display the cart contents meta box.
	 *
	 * @since 2.0.0
	 *
	 * @param WP_Post $post The post object.
	 */
	public function display_meta_box( $post ) {

		echo '<pre>';
		print_r( get_post_meta( $post->ID, 'wpf_cart_contents', true ) );
		echo '</pre>';
	}

	/**
	 * Adds cron schedule for every 5 minutes.
	 *
	 * @since 1.9.0
	 *
	 * @param array $schedules The cron schedules.
	 */
	public function process_abandoned_carts_cron_schedule( $schedules ) {

		$schedules['wpf_fifteen_minutes'] = array(
			'interval' => MINUTE_IN_SECONDS * 15,
			'display'  => __( 'WP Fusion Abandoned Carts Worker', 'wp-fusion-abandoned-cart' ),
		);

		return $schedules;
	}

	/**
	 * Create or update a cart in the DB.
	 *
	 * @since  1.7.0
	 *
	 * @param  array  $cart_contents The cart contents.
	 * @param  string $contact_id    The contact ID.
	 * @return int|WP_Error The cart ID or an error.
	 */
	public function create_update_cart( $cart_contents, $contact_id ) {

		if ( isset( $cart_contents['update_data'] ) && ! isset( $cart_contents['user_data'] ) ) {
			$cart_contents['user_data'] = $cart_contents['update_data'];
		}

		$contact_id = str_replace( '+', '-', $contact_id ); // Sendinblue uses email addresses as contact IDs.
		$contact_id = sanitize_text_field( $contact_id );

		$cart = get_page_by_path( 'wp-fusion-abandoned-cart-' . $contact_id, OBJECT, 'wpf_cart' );

		// Merge with the existing data so we don't lose any order IDs.

		if ( ! empty( $cart ) ) {

			$content = get_post_meta( $cart->ID, 'wpf_cart_contents', true );

			if ( empty( $content ) ) {
				$content = array();
			}

			$cart_contents = array_merge( $content, $cart_contents );

			// So the post_modified is updated.
			$cart_id = wp_update_post( array( 'ID' => $cart->ID ) );

		} else {

			$cart = array(
				'post_title'  => 'wp-fusion-abandoned-cart-' . $contact_id,
				'post_status' => 'publish',
				'post_type'   => 'wpf_cart',
			);

			$cart_id = wp_insert_post( $cart, true, false ); // don't fire after insert hooks.

			wpf_log( 'info', wpf_get_current_user_id(), 'Saved cart <a href="' . admin_url( 'post.php?action=edit&post=' . $cart_id ) . '">#' . $cart_id . '</a> for contact #' . $contact_id, array( 'source' => 'wpf-abandoned-cart' ) );

		}

		if ( wpf_get_option( 'abandoned_cart_delay' ) ) {
			update_post_meta( $cart_id, 'wpf_cart_pending', true );
		}

		update_post_meta( $cart_id, 'wpf_cart_contents', $cart_contents );

		return $cart_id;
	}

	/**
	 * Get the cart for a contact ID.
	 *
	 * @since  1.7.0
	 *
	 * @param  string $contact_id The contact ID.
	 * @return array|bool The cart, or false if not found.
	 */
	public function get_cart( $contact_id ) {

		$cart = get_page_by_path( 'wp-fusion-abandoned-cart-' . $contact_id, OBJECT, 'wpf_cart' );

		if ( null === $cart ) {
			return false;
		}

		return get_post_meta( $cart->ID, 'wpf_cart_contents', true );
	}

	/**
	 * Check if a contact's cart is pending sync, using the delay option.
	 *
	 * @since 1.9.0
	 * @since 2.0.0 Added optional email parameter.
	 *
	 * @param  string $contact_id    The contact ID.
	 * @param  string $email_address The email address.
	 * @return bool   Whether or not the cart is pending.
	 */
	public function is_cart_pending( $contact_id, $email_address = '' ) {

		$cart = get_page_by_path( 'wp-fusion-abandoned-cart-' . $contact_id, OBJECT, 'wpf_cart' );

		if ( null === $cart && $email_address ) {
			$contact_id = md5( sanitize_email( $email_address ) );
			$cart       = get_page_by_path( 'wp-fusion-abandoned-cart-' . $contact_id, OBJECT, 'wpf_cart' );
		}

		if ( null === $cart ) {
			return false;
		}

		return (bool) get_post_meta( $cart->ID, 'wpf_cart_pending', true );
	}

	/**
	 * Delete a cart after checkout.
	 *
	 * @since 1.7.0
	 * @since 2.0.0 Added optional email parameter.
	 *
	 * @param  string $contact_id    The contact ID.
	 * @param  string $email_address The email address.
	 * @return bool   Whether or not the cart was deleted.
	 */
	public function delete_cart( $contact_id, $email_address = '' ) {

		$cart = get_page_by_path( 'wp-fusion-abandoned-cart-' . $contact_id, OBJECT, 'wpf_cart' );

		if ( null === $cart && $email_address ) {
			$contact_id = md5( sanitize_email( $email_address ) );
			$cart       = get_page_by_path( 'wp-fusion-abandoned-cart-' . $contact_id, OBJECT, 'wpf_cart' );
		}

		if ( null !== $cart ) {
			remove_all_actions( 'before_delete_post' ); // stop other plugins trying to handle this.
			return wp_delete_post( $cart->ID, true );
		} else {
			return true;
		}
	}

	/**
	 * Process abandoned carts.
	 *
	 * @since 1.7.0
	 */
	public function process_abandoned_carts() {

		$delay = wpf_get_option( 'abandoned_cart_delay' );

		$args = array(
			'fields'         => 'ids',
			'posts_per_page' => 100,
			'post_type'      => 'wpf_cart',
			'date_query'     => array(
				'before' => gmdate( 'Y-m-d H:i:s', strtotime( '-' . $delay . ' minutes' ) ),
				'column' => 'post_date_gmt',
			),
			'meta_query'     => array(
				array(
					'key'     => 'wpf_cart_pending',
					'compare' => 'EXISTS',
				),
			),
		);

		foreach ( get_posts( $args ) as $post_id ) {

			$cart_contents = get_post_meta( $post_id, 'wpf_cart_contents', true );

			if ( isset( $cart_contents['source'] ) && isset( wp_fusion_abandoned_cart()->integrations->{ $cart_contents['source'] } ) ) {

				$email_hash = md5( sanitize_email( $cart_contents['user_email'] ) );

				$sync_fields = true;

				if ( $email_hash === $cart_contents['contact_id'] ) {

					// There was no contact created yet, let's create one now.

					$update_data = array_merge( $cart_contents['update_data'], $cart_contents['user_data'] );

					$contact_id = wp_fusion()->crm->get_contact_id( $cart_contents['user_email'] );

					if ( empty( $contact_id ) ) {

						wpf_log(
							'info',
							wpf_get_current_user_id(),
							'Abandoned cart adding contact:',
							array(
								'meta_array' => $update_data,
								'source'     => 'wpf-abandoned-cart',
							)
						);

						$contact_id = wp_fusion()->crm->add_contact( $update_data );

					} elseif ( ! is_wp_error( $contact_id ) ) {

						wpf_log(
							'info',
							wpf_get_current_user_id(),
							'Abandoned cart updating contact #' . $contact_id . ':',
							array(
								'meta_array' => $update_data,
								'source'     => 'wpf-abandoned-cart',
							)
						);

						$contact_id = wp_fusion()->crm->update_contact( $contact_id, $update_data );

					}

					if ( is_wp_error( $contact_id ) ) {
						wpf_log( 'error', wpf_get_current_user_id(), 'Abandoned cart adding contact failed: ' . $contact_id->get_error_message(), array( 'source' => 'wpf-abandoned-cart' ) );
						delete_post_meta( $post_id, 'wpf_cart_pending' );
						continue;
					}

					$cart_contents['contact_id'] = $contact_id;

					do_action( 'wpf_guest_contact_created', $cart_contents['contact_id'], $cart_contents['user_email'] );

					// Change the cart post title to reflect the new contact ID.
					wp_update_post(
						array(
							'ID'         => $post_id,
							'post_name'  => 'wp-fusion-abandoned-cart-' . $cart_contents['contact_id'],
							'post_title' => 'wp-fusion-abandoned-cart-' . $cart_contents['contact_id'],
						)
					);

					// Update the recovery URL to use the contact ID instead of hashed email.

					if ( ! empty( $cart_contents['recovery_url'] ) ) {
						$cart_contents['recovery_url']                     = str_replace( $email_hash, $cart_contents['contact_id'], $cart_contents['recovery_url'] );
						$cart_contents['update_data']['cart_recovery_url'] = $cart_contents['recovery_url'];
					}

					update_post_meta( $post_id, 'wpf_cart_contents', $cart_contents );

					// We don't need to sync the $update_data a second time.
					$sync_fields = false;

				}

				wp_fusion_abandoned_cart()->integrations->{ $cart_contents['source'] }->process_abandoned_cart( $cart_contents['contact_id'], $cart_contents, $sync_fields );

				// Un-mark the cart as pending.
				delete_post_meta( $post_id, 'wpf_cart_pending' );
			}
		}
	}

	/**
	 * Auto-delete the cached cart data after 30 days.
	 *
	 * @since 1.7.0
	 */
	public function cleanup() {

		$args = array(
			'fields'         => 'ids',
			'posts_per_page' => 200,
			'post_type'      => 'wpf_cart',
			'date_query'     => array(
				'before' => gmdate( 'Y-m-d', strtotime( '-30 days' ) ),
			),
		);

		$posts = get_posts( $args );

		foreach ( $posts as $post_id ) {
			wp_delete_post( $post_id, true );
		}
	}
}
