<?php

class WPF_Abandoned_Cart_ActiveCampaign {

	/**
	 * Lets the rest of WP Fusion know which features are supported by the CRM
	 */

	public $supports = array( 'add_cart' );

	/**
	 * Get things started
	 *
	 * @since 1.4.0
	 */
	public function init() {

		add_action( 'wpf_abandoned_cart_created', array( $this, 'cart_created' ), 10, 2 );
	}

	/**
	 * Adds the cart to ActiveCampaign
	 *
	 * @since 1.4.0
	 *
	 * @param string $contact_id The contact ID.
	 * @param array $args        The cart data.
	 * @return int|false|WP_Error The cart ID or an error.
	 */

	public function cart_created( $contact_id, $args ) {

		if ( ! wpf_get_option( 'abandoned_cart_sync_carts' ) ) {
			return false;
		}

		$connection_id = wp_fusion()->crm->get_connection_id();
		$customer_id   = wp_fusion()->crm->get_customer_id( $contact_id, $connection_id );

		$product_objects = array();

		$calc_totals = 0;

		foreach ( $args['items'] as $item ) {

			$product_data = array(
				'externalid' => $item['product_id'],
				'name'       => $item['name'],
				'price'      => floatval( $item['price'] ) * 100,
				'quantity'   => $item['quantity'],
				'productUrl' => $item['product_url'],
			);

			if ( ! empty( $item['image_url'] ) ) {
				$product_data['imageUrl'] = $item['image_url'];
			}

			if ( ! empty( $item['description'] ) ) {
				$product_data['description'] = $item['description'];
			}

			$product_objects[] = (object) $product_data;

			$calc_totals += $item['price'] * $item['quantity'];

		}

		$body = array(
			'ecomOrder' => array(
				'externalcheckoutid'  => $args['cart_id'],
				'source'              => 1,
				'email'               => $args['user_email'],
				'orderProducts'       => $product_objects,
				'orderUrl'            => $args['recovery_url'],
				'abandonedDate'       => wpf_get_iso8601_date(),
				'externalCreatedDate' => wpf_get_iso8601_date(),
				'totalPrice'          => $calc_totals * 100,
				'currency'            => $args['currency'],
				'connectionid'        => $connection_id,
				'customerid'          => $customer_id,
			),
		);

		if ( ! empty( $args['shipping'] ) && ! empty( $args['shipping']['total'] ) ) {
			$body['ecomOrder']['shippingAmount'] = $args['shipping']['total'] * 100;
			$body['ecomOrder']['shippingMethod'] = $args['shipping']['method'];
		}

		// Add in discounts if applicable

		if ( ! empty( $args['discounts'] ) ) {
			foreach ( $args['discounts'] as $discount ) {

				if ( ! isset( $body['ecomOrder']['orderDiscounts'] ) ) {
					$body['ecomOrder']['orderDiscounts'] = array();
					$body['ecomOrder']['discountAmount'] = 0;
				}

				$body['ecomOrder']['orderDiscounts'][] = array(
					'name'           => $discount['name'],
					'type'           => 'order',
					'discountAmount' => $discount['amount'] * 100,
				);

				$body['ecomOrder']['discountAmount'] += ( $discount['amount'] * 100 );

				if ( 'WooCommerce' === $args['provider'] ) {
					// For now the EDD prices will include the discount (for Impress) and the Woo prices will not
					$body['ecomOrder']['totalPrice'] -= ( $discount['amount'] * 100 );
				}
			}
		}

		$params         = wp_fusion()->crm->get_params();
		$params['body'] = wp_json_encode( $body );

		$api_url = wpf_get_option( 'ac_url' );

		if ( false === $args['update'] ) {

			// Create a new cart.

			wpf_log(
				'info',
				wpf_get_current_user_id(),
				'Syncing cart to ActiveCampaign:',
				array(
					'meta_array_nofilter' => $body,
					'source'              => 'wpf-abandoned-cart',
				)
			);

			$request  = $api_url . '/api/3/ecomOrders';
			$response = wp_remote_post( $request, $params );

			if ( is_wp_error( $response ) ) {

				if ( 'duplicate' === $response->get_error_code() ) {

					$ecom_order_id = $this->get_order_id( $args['cart_id'] );

					if ( false !== $ecom_order_id ) {

						// Update the saved cart.
						$cart = array(
							'order_id'    => $ecom_order_id,
							'customer_id' => $customer_id,
						);

						wp_fusion_abandoned_cart()->carts->create_update_cart( $cart, $contact_id );

						$args['update'] = true;

						// Try again updating the cart.
						return $this->cart_created( $contact_id, $args );

					}
				}

				wpf_log( $response->get_error_code(), wpf_get_current_user_id(), 'Error adding cart: ' . $response->get_error_message(), array( 'source' => 'wpf-abandoned-cart' ) );
				return $response;

			}

			$response = json_decode( wp_remote_retrieve_body( $response ) );

			$order_id = $response->{'ecomOrder'}->id;

			// Update the saved cart
			$cart = array(
				'order_id'    => $order_id,
				'customer_id' => $customer_id,
			);

			wp_fusion_abandoned_cart()->carts->create_update_cart( $cart, $contact_id );

			return $order_id;

		} else {

			// Update an existing cart.

			$cart = wp_fusion_abandoned_cart()->carts->get_cart( $contact_id );

			if ( empty( $cart ) || empty( $cart['order_id'] ) ) {

				wpf_log( 'notice', wpf_get_current_user_id(), 'Unable to update cart for contact ID ' . $contact_id . ': couldn\'t retrieve ActiveCampaign order ID.', array( 'source' => 'wpf-abandoned-cart' ) );

				// Try again.
				$args['update'] = false;

				return $this->cart_created( $contact_id, $args );

			}

			// Handle emptied carts.

			if ( empty( $product_objects ) ) {

				$action           = 'Deleting';
				$params['method'] = 'DELETE';

				wp_fusion_abandoned_cart()->carts->delete_cart( $contact_id );

			} else {
				$action           = 'Updating';
				$params['method'] = 'PUT';
			}

			wpf_log(
				'info',
				wpf_get_current_user_id(),
				$action . ' cart #' . $cart['order_id'] . ' in ActiveCampaign:',
				array(
					'meta_array_nofilter' => $body,
					'source'              => 'wpf-abandoned-cart',
				)
			);

			$request  = $api_url . '/api/3/ecomOrders/' . $cart['order_id'];
			$response = wp_remote_request( $request, $params );

			if ( is_wp_error( $response ) ) {

				wpf_log( $response->get_error_code(), wpf_get_current_user_id(), 'Error updating cart: ' . $response->get_error_message(), array( 'source' => 'wpf-abandoned-cart' ) );
				return $response;

			}

			$response = json_decode( wp_remote_retrieve_body( $response ) );
			$order_id = $response->{'ecomOrder'}->id;

			return $order_id;

		}
	}

	/**
	 * Searches for a deep data order by "externalid", i.e. the WooCommerce session ID.
	 *
	 * @since 1.7.3
	 *
	 * @param int $order_id The order ID.
	 * @return int|false|WP_Error The cart ID, false, or WP_Error.
	 */
	public function get_order_id( $order_id ) {

		$params   = wp_fusion()->crm->get_params();
		$response = wp_remote_get( wp_fusion()->crm->api_url . 'api/3/ecomOrders/?filters[externalcheckoutid]=' . $order_id, $params );

		if ( is_wp_error( $response ) ) {
			return $response;
		}

		$response = json_decode( wp_remote_retrieve_body( $response ) );

		if ( empty( $response->{'ecomOrders'} ) ) {
			return false;
		}

		return $response->ecomOrders[0]->id; // @phpcs:ignore
	}
}
