<?php

/**
 * Handles shared functionality between the abandoned cart integrations.
 *
 * @since 1.0.0
 */
class WPF_Abandoned_Cart_Integrations_Base {

	/**
	 * The slug for the integration.
	 *
	 * @since 1.0.0
	 * @var string
	 */
	public $slug = '';

	/**
	 * Get things started.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {

		$this->init();

		if ( $this->slug ) {
			// Make the integration globally accessible.
			wp_fusion_abandoned_cart()->integrations->{ $this->slug } = $this;
		}

		add_action( "wp_ajax_nopriv_wpf_abandoned_cart_{$this->slug}", array( $this, 'save_checkout_data' ) );
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
	}

	/**
	 * Registers hooks unique to the integration.
	 *
	 * @since 1.0.0
	 */
	public function init() {}

	/**
	 * Handles updating the contact and applying checkout tags.
	 *
	 * @since 2.0.0
	 *
	 * @param bool|string $contact_id The contact ID, or false if unknown.
	 * @param array       $user_data  POST data from checkout.
	 */
	public function checkout_begin( $contact_id = false, $user_data = array() ) {}

	/**
	 * Formats the POSTed form data to get relevant fields for each integration.
	 *
	 * Can override in the integration to check for alternate input names.
	 *
	 * @since 2.0.0
	 *
	 * @param array $post_data The POST data.
	 * @return array The formatted data.
	 */
	public function format_posted_data( $post_data ) {
		return $post_data;
	}

	/**
	 * Get desired field from POSTed data.
	 *
	 * @since 2.0.0
	 *
	 * @param array  $data       The POSTed form data.
	 * @param string $field_name The field name to search for.
	 * @return mixed|null The field value.
	 */
	protected function get_field_from_data( $data, $field_name ) {

		if ( isset( $data[ $field_name ] ) ) {
			return $data[ $field_name ];
		}

		foreach ( $data as $key => $value ) {
			if ( strpos( $key, $field_name ) !== false ) {
				return $value;
			}
		}

		return null;
	}

	/**
	 * Saves data entered at checkout to CRM
	 *
	 * @access public
	 * @return void
	 */
	public function save_checkout_data() {

		check_ajax_referer( 'wpf-abandoned-cart', 'nonce' );

		$post_data = wpf_clean( wp_unslash( $_POST ) );

		$post_data = $this->format_posted_data( $post_data );

		if ( isset( $post_data['email_optin'] ) ) {
			unset( $post_data['email_optin'] ); // we don't want to sync this until they've actually submitted the form.
		}

		// Check to see if it's an existing registered user.
		$user = get_user_by( 'email', $post_data['user_email'] );

		$contact_id = false;

		if ( is_object( $user ) ) {
			$contact_id = wpf_get_contact_id( $user->ID );
		}

		// See if there's a contact in the CRM.
		if ( ! $contact_id ) {
			$contact_id = wp_fusion()->crm->get_contact_id( $post_data['user_email'] );
		}

		if ( ! is_wp_error( $contact_id ) && empty( $contact_id ) ) {

			if ( wpf_get_option( 'abandoned_cart_delay' ) ) {

				// If the delay is enabled, we'll generate a random ID to use to reference the cart.
				$contact_id = md5( sanitize_email( $post_data['user_email'] ) );

			} else {
				wpf_log(
					'info',
					wpf_get_current_user_id(),
					'Abandoned cart adding contact:',
					array(
						'meta_array' => $post_data,
						'source'     => 'wpf-abandoned-cart',
					)
				);

				$contact_id = wp_fusion()->crm->add_contact( $post_data );

				if ( ! is_wp_error( $contact_id ) ) {
					do_action( 'wpf_guest_contact_created', $contact_id, $post_data['user_email'] );
				}
			}
		}

		if ( is_wp_error( $contact_id ) ) {

			wpf_log( 'error', wpf_get_current_user_id(), 'Error adding contact: ' . $contact_id->get_error_message(), array( 'source' => 'wpf-abandoned-cart' ) );
			wp_send_json_error( $contact_id );

		}

		// We don't use this action anymore but we'll leave it in case anyone else is using it.
		do_action( 'wpf_abandoned_cart_start', $contact_id, array(), $post_data, $post_data['source'] );

		// Call the integrations to process their data:
		$this->checkout_begin( $contact_id, $post_data );

		wp_send_json_success( $contact_id );
	}

	/**
	 * Determines if abandoned cartscripts should be enqueued.
	 *
	 * Override in the integration depending on where the checkout page is.
	 *
	 * @since 2.0.0
	 *
	 * @return bool True if scripts should be enqueued, false otherwise.
	 */
	protected function should_enqueue_scripts() {
		return false;
	}

	/**
	 * Enqueue scripts on checkout page.
	 *
	 * @since 2.0.0
	 */
	public function enqueue_scripts() {

		if ( $this->should_enqueue_scripts() ) {

			$data = array(
				'ajaxurl'             => admin_url( 'admin-ajax.php' ),
				'nonce'               => wp_create_nonce( 'wpf-abandoned-cart' ),
				'on_submit'           => wpf_get_option( 'abandoned_cart_on_submit' ),
				'progressive_updates' => wpf_get_option( 'abandoned_cart_progressive_updates' ),
			);

			wp_enqueue_script( 'wpf-abandoned-cart', WPF_ABANDONED_CART_DIR_URL . 'assets/wpf-abandoned-cart.js', array( 'jquery' ), WPF_ABANDONED_CART_VERSION, true );
			wp_localize_script( 'wpf-abandoned-cart', 'wpf_ac_data', $data );
		}
	}
}
