<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

class WPF_Abandoned_Cart_LifterLMS extends WPF_Abandoned_Cart_Integrations_Base {

	/**
	 * The slug for the integration.
	 *
	 * @since 1.6.0
	 * @var string
	 */
	public $slug = 'lifterlms';

	/**
	 * Get things started
	 *
	 * @access public
	 * @return void
	 */

	public function init() {

		// Abandoned cart tracking.
		add_action( 'lifterlms_pre_checkout_form', array( $this, 'pre_checkout_form' ) );

		// Remove the tags after checkout complete
		add_action( 'lifterlms_access_plan_purchased', array( $this, 'checkout_complete' ), 10, 2 );

		// Meta Box
		add_action( 'llms_access_plan_mb_after_row_five', array( $this, 'access_plan_settings' ), 10, 3 );
		add_action( 'llms_access_plan_saved', array( $this, 'save_plan' ), 10, 3 );
	}

	/**
	 * Enqueue scripts on checkout page.
	 *
	 * @since 2.0.0
	 * @return bool Whether to enqueue scripts.
	 */
	public function should_enqueue_scripts() {

		if ( is_llms_checkout() && ! wpf_is_user_logged_in() ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Start action for logged-in users
	 *
	 * @access public
	 * @return void
	 */

	public function pre_checkout_form() {

		if ( ! wpf_is_user_logged_in() ) {
			return;
		}

		$contact_id = wp_fusion()->user->get_contact_id();

		$user                    = wp_get_current_user();
		$user_data['user_email'] = $user->user_email;
		$user_data['first_name'] = $user->first_name;
		$user_data['last_name']  = $user->last_name;

		$this->checkout_begin( $contact_id, $user_data );

	}

	/**
	 * Apply abandoned cart tags
	 *
	 * @access public
	 * @return void
	 */

	public function checkout_begin( $contact_id = false, $user_data = array() ) {

		$plan_id = $this->get_plan_id();

		// If we can't find the access plan, quit
		if ( empty( $plan_id ) || 'llms_access_plan' !== get_post_type( $plan_id ) ) {
			return;
		}

		$apply_tags = wpf_get_option( 'abandoned_cart_apply_tags', array() );

		$settings = get_post_meta( $plan_id, '_wpf_abandoned_cart', true );

		if ( ! empty( $settings ) && ! empty( $settings['apply_tags_abandoned'] ) ) {
			$apply_tags = array_merge( $apply_tags, $settings['apply_tags_abandoned'] );
		}

		$items   = array();
		$items[] = $this->get_checkout_items( $plan_id );

		$update_data = array();
		// Recovery URL
		$recovery_url = $this->get_cart_recovery_url( $contact_id, $user_data );
		if ( wpf_is_field_active( 'cart_recovery_url' ) ) {
			$update_data['cart_recovery_url'] = $recovery_url;
		}

		// Cart value
		if ( wpf_is_field_active( 'cart_value' ) ) {
			$update_data['cart_value'] = $plan->get( 'price' );
		}

		// Discounts (To be added later in wpfusion)
		$discounts = $this->get_cart_discounts();
		if ( ! empty( $discounts ) ) {

			if ( wpf_is_field_active( 'llms_cart_discount_code' ) ) {
				$update_data['llms_cart_discount_code'] = $discounts[0]['name'];
			}

			if ( wpf_is_field_active( 'llms_cart_discount_amount' ) ) {
				$update_data['llms_cart_discount_amount'] = $discounts[0]['amount'];
			}
		}

		$update_data = apply_filters( 'wpf_abandoned_cart_data', $update_data, $contact_id );

		$args = array(
			'source'       => $this->slug,
			'contact_id'   => $contact_id,
			'update_data'  => $update_data,
			'apply_tags'   => $apply_tags,
			'cart_id'      => rand(),
			'recovery_url' => $recovery_url,
			'items'        => $items,
			'discounts'    => $this->get_cart_discounts(),
			'user_email'   => $user_data['user_email'],
			'provider'     => 'LifterLMS',
			'update'       => false,
			'currency'     => get_lifterlms_currency(),
		);

		if ( ! wpf_get_option( 'abandoned_cart_delay' ) ) {
			$this->process_abandoned_cart( $contact_id, $args );
		} else {
			wp_fusion_abandoned_cart()->carts->create_update_cart( $args, $contact_id );
		}

	}

	/**
	 * Get plan ID
	 *
	 * @return integer
	 */
	private function get_plan_id() {

		if ( isset( $_GET['plan'] ) ) {

			$plan_id = $_GET['plan'];

		} elseif ( isset( $_SERVER['HTTP_REFERER'] ) ) {

			// Get the plan ID in an AJAX request (not pretty, I know)
			$checkout_url = $_SERVER['HTTP_REFERER'];
			$parts        = parse_url( $checkout_url );
			parse_str( $parts['query'], $query );

			if ( isset( $query['plan'] ) ) {
				$plan_id = $query['plan'];
			}
		}

		return $plan_id;
	}

	/**
	 * Get checkout items
	 *
	 * @param integer $plan_id
	 * @return array
	 */
	private function get_checkout_items( $plan_id ) {

		$plan = new LLMS_Access_Plan( $plan_id );

		return array(
			'product_id'  => $plan->get( 'id' ),
			'name'        => $plan->get( 'title' ),
			'quantity'    => 1,
			'product_url' => get_the_permalink( $plan->get( 'product_id' ) ),
			'price'       => $plan->get( 'price' ),
			'total'       => $plan->get( 'price' ),
			'description' => $plan->get( 'content' ),
		);
	}

	/**
	 * Syncs the data and applies the tags for the cart, either in real time or after
	 * a delay, depending on the settings.
	 *
	 * @since 2.0.0
	 *
	 * @param string|bool $contact_id  The contact ID.
	 * @param array       $args        The cart data.
	 * @param bool        $sync_fields Whether to sync fields.
	 */
	public function process_abandoned_cart( $contact_id, $args, $sync_fields = true ) {

		if ( $sync_fields && ! empty( wp_fusion()->crm->map_meta_fields( $args['update_data'] ) ) ) {

			wpf_log(
				'info',
				wpf_get_user_id( $contact_id ),
				'Syncing abandoned cart data:',
				array(
					'meta_array_nofilter' => $args['update_data'],
					'source'              => 'wpf-abandoned-cart',
				)
			);

			wp_fusion()->crm->update_contact( $contact_id, $args['update_data'] );

		}

		if ( ! empty( $args['apply_tags'] ) ) {

			// Apply the tags
			if ( wpf_get_contact_id() === $contact_id ) {

				wp_fusion()->user->apply_tags( $args['apply_tags'] );

			} elseif ( false !== $contact_id ) {

				wpf_log(
					'info',
					wpf_get_user_id( $contact_id ),
					'Applying abandoned cart tags:',
					array(
						'tag_array' => $args['apply_tags'],
						'source'    => 'wpf-abandoned-cart',
					)
				);

				wp_fusion()->crm->apply_tags( $args['apply_tags'], $contact_id );

			}
		}

		do_action( 'wpf_abandoned_cart_created', $contact_id, $args );

	}


	/**
	 * Get cart discounts.
	 *
	 * @return array
	 */
	public function get_cart_discounts() {

		if ( ! llms()->session ) {
			return;
		}

		$coupon_session = llms()->session->get( 'llms_coupon', false );
		$coupon         = new LLMS_Coupon( $coupon_session['coupon_id'] );

		$plan_id       = $this->get_plan_id();
		$plan          = new LLMS_Access_Plan( $plan_id );
		$price         = $plan->get( 'price' );
		$coupon_amount = $plan->get_price_with_coupon( 'price', $coupon_session['coupon_id'], false, false );

		$discounts[] = array(
			'name'   => $coupon->get( 'title' ),
			'amount' => (float) $price - (float) $coupon_amount,
		);

		return $discounts;
	}



	/**
	 * Get cart recovery URL
	 *
	 * @access public
	 * @return string Recovery URL
	 */

	public function get_cart_recovery_url( $contact_id, $user_data ) {
		$contents = array();
		$plan_id  = $this->get_plan_id();
		if ( $plan_id ) {
			$contents[] = $this->get_checkout_items( $plan_id );
		}

		$session = array(
			'user_data' => $user_data,
			'contents'  => $contents,
			'discounts' => $this->get_cart_discounts(),
			'source'    => 'lifterlms',
		);

		wp_fusion_abandoned_cart()->carts->create_update_cart( $session, $contact_id );
		$checkout_url = get_page( get_option( 'lifterlms_checkout_page_id' ) );

		$url = add_query_arg( 'wpfrc', urlencode( $contact_id ), $checkout_url->guid );

		return $url;

	}


	/**
	 * Remove abandoned cart tags
	 *
	 * @access public
	 * @return void
	 */

	public function checkout_complete( $user_id, $plan_id ) {

		$contact_id = wpf_get_contact_id( $user_id );
		$user       = get_userdata( $user_id );

		if ( wp_fusion_abandoned_cart()->carts->is_cart_pending( $contact_id, $user->user_email ) ) {
			wp_fusion_abandoned_cart()->carts->delete_cart( $contact_id, $user->user_email );
			return; // Cart hasn't been synced yet, don't need to do anything.
		}

		// Get tags to be removed
		$remove_tags = wpf_get_option( 'abandoned_cart_apply_tags' );

		if ( empty( $remove_tags ) ) {
			$remove_tags = array();
		}

		$settings = get_post_meta( $plan_id, '_wpf_abandoned_cart', true );

		if ( ! empty( $settings ) && ! empty( $settings['apply_tags_abandoned'] ) ) {
			$remove_tags = array_merge( $remove_tags, $settings['apply_tags_abandoned'] );
		}

		if ( ! empty( $remove_tags ) ) {
			wp_fusion()->user->remove_tags( $remove_tags, $user_id );
		}

	}


	/*
	 * Adds WPF settings to LLMS access plan meta box
	 *
	 * @access  public
	 * @return  mixed Access Plan Settings
	 */

	public function access_plan_settings( $plan, $id, $order ) {

		if ( empty( $plan ) ) {
			return;
		}

		?>
		<div class="llms-metabox-field d-1of3">

			<label><?php _e( 'Apply Tags - Abandoned Cart', 'wp-fusion-abandoned-cart' ); ?></label>
			<?php

			$settings = get_post_meta( $plan->id, '_wpf_abandoned_cart', true );

			if ( empty( $settings ) ) {
				$settings = array( 'apply_tags_abandoned' => array() );
			}

			$args = array(
				'setting'   => $settings['apply_tags_abandoned'],
				'meta_name' => "_llms_plans[{$order}][apply_tags_abandoned]",
			);

			wpf_render_tag_multiselect( $args );

			?>
		</div>

		<?php

	}

	/**
	 * Save access plan
	 *
	 * @access  public
	 * @return  void
	 */

	public function save_plan( $plan, $raw_plan_data, $metabox ) {

		if ( ! empty( $raw_plan_data['apply_tags_abandoned'] ) ) {

			update_post_meta( $raw_plan_data['id'], '_wpf_abandoned_cart', array( 'apply_tags_abandoned' => $raw_plan_data['apply_tags_abandoned'] ) );

		} else {

			delete_post_meta( $raw_plan_data['id'], '_wpf_abandoned_cart' );

		}

	}


}

new WPF_Abandoned_Cart_LifterLMS();
