<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

class WPF_Abandoned_Cart_Memberpress extends WPF_Abandoned_Cart_Integrations_Base {

	/**
	 * The slug for the integration.
	 *
	 * @since x.x.x
	 * @var string
	 */
	public $slug = 'memberpress';

	/**
	 * Get things started
	 *
	 * @access public
	 * @return void
	 */

	public function init() {

		// Abandoned cart tracking.
		add_action( 'mepr-txn-status-pending', array( $this, 'checkout_begin_loggedin' ) );

		// After checkout complete (MemberPress has a ton of hooks for this and it makes no sense)
		add_action( 'mepr-event-transaction-completed', array( $this, 'checkout_complete' ) );
		add_action( 'mepr-txn-status-complete', array( $this, 'checkout_complete' ) );
		add_action( 'mepr-signup', array( $this, 'checkout_complete' ) );
		add_action( 'mepr-event-subscription-created', array( $this, 'checkout_complete' ) );

		add_action( 'wpf_memberpress_meta_box', array( $this, 'meta_box' ), 10, 2 );

	}

	/**
	 * Enqueue scripts on checkout page.
	 *
	 * @since 2.0.0
	 * @return bool Whether to enqueue scripts.
	 */
	public function should_enqueue_scripts() {

		if ( MeprProduct::is_product_page( get_post() ) && ! wpf_is_user_logged_in() ) {
			return true;
		} else {
			return false;
		}

	}

	/**
	 * Maybe apply abandoned cart tags for registered users when a pending transaction is created
	 *
	 * @access public
	 * @return void
	 */

	public function checkout_begin_loggedin( $txn ) {

		if ( ! wpf_is_user_logged_in() ) {
			return;
		}

		$current_user = wp_get_current_user();

		$user_data = array(
			'first_name' => $current_user->first_name,
			'last_name'  => $current_user->last_name,
			'user_email' => $current_user->user_email,
		);

		$this->checkout_begin( wpf_get_contact_id(), $user_data );

	}

	/**
	 * Apply abandoned cart tags
	 *
	 * @access public
	 * @return void
	 */

	public function checkout_begin( $contact_id = false, $user_data = array() ) {

		$apply_tags = wpf_get_option( 'abandoned_cart_apply_tags', array() );

		// Guests use product_id, logged in users use mepr_product_id.
		$membership_id = isset( $_POST['product_id'] ) ? intval( $_POST['product_id'] ) : intval( $_POST['mepr_product_id'] );

		$settings = get_post_meta( $membership_id, 'wpf-settings-memberpress', true );

		if ( ! empty( $settings ) && ! empty( $settings['apply_tags_abandoned_cart'] ) ) {
			$apply_tags = array_merge( $apply_tags, $settings['apply_tags_abandoned_cart'] );
		}

		$update_data = array();
		// Recovery URL
		$recovery_url = $this->get_cart_recovery_url( $contact_id, $user_data );
		if ( wpf_is_field_active( 'cart_recovery_url' ) ) {
			$update_data['cart_recovery_url'] = $recovery_url;
		}

		// Discounts
		$discounts = $this->get_cart_discounts( $user_data );
		if ( ! empty( $discounts ) ) {

			if ( wpf_is_field_active( 'mepr_coupon' ) ) {
				$update_data['mepr_coupon'] = $discounts[0]['name'];
			}
		}

		$items = $this->get_cart_items( $membership_id );

		$mepr_options = MeprOptions::fetch();

		$apply_tags = apply_filters( 'wpf_abandoned_cart_apply_tags', $apply_tags, $contact_id );

		$args = array(
			'source'       => $this->slug,
			'contact_id'   => $contact_id,
			'user_data'    => $user_data,
			'update_data'  => $update_data,
			'apply_tags'   => $apply_tags,
			'cart_id'      => rand(),
			'recovery_url' => $recovery_url,
			'items'        => $items,
			'user_email'   => $user_data['user_email'],
			'provider'     => 'MemberPress',
			'update'       => false,
			'currency'     => $mepr_options->currency_code,
		);

		if ( ! wpf_get_option( 'abandoned_cart_delay' ) ) {
			$this->process_abandoned_cart( $contact_id, $args );
		} else {
			wp_fusion_abandoned_cart()->carts->create_update_cart( $args, $contact_id );
		}

	}


	/**
	 * Get cart discounts.
	 *
	 * @return array
	 */
	public function get_cart_discounts( $data ) {
		$coupon = $data['mepr_coupon_code'] ?? '';

		if ( empty( $coupon ) ) {
			return;
		}

		$discounts[] = array(
			'name' => $coupon,
		);

		return $discounts;
	}


	/**
	 * Get cart items.
	 *
	 * @since 1.5.0
	 *
	 * @param int $membership_id The membership ID.
	 * @return array The items.
	 */
	private function get_cart_items( $membership_id ) {
		$items = array(
			array(
				'product_id'  => $membership_id,
				'name'        => get_post_field( 'post_title', $membership_id, 'raw' ),
				'quantity'    => 1,
				'price'       => get_post_meta( $membership_id, '_mepr_product_price', true ),
				'total'       => get_post_meta( $membership_id, '_mepr_product_price', true ),
				'product_url' => get_the_permalink( $membership_id ),
			),
		);

		$size = wpf_get_option( 'abandoned_cart_image_size', 'medium' );

		$image = wp_get_attachment_image_src( get_post_thumbnail_id( $membership_id ), $size );

		if ( ! empty( $image ) ) {
			$items[0]['image_url'] = $image[0];
		}

		return $items;
	}

	/**
	 * Syncs the data and applies the tags for the cart, either in real time or after
	 * a delay, depending on the settings.
	 *
	 * @param string|bool $contact_id  The contact ID.
	 * @param array       $args        The cart data.
	 * @param bool        $sync_fields Whether to sync fields.
	 */
	public function process_abandoned_cart( $contact_id, $args, $sync_fields = true ) {

		if ( $sync_fields && ! empty( wp_fusion()->crm->map_meta_fields( $args['update_data'] ) ) ) {

			wpf_log(
				'info',
				wpf_get_user_id( $contact_id ),
				'Syncing abandoned cart data:',
				array(
					'meta_array_nofilter' => $args['update_data'],
					'source'              => 'wpf-abandoned-cart',
				)
			);

			wp_fusion()->crm->update_contact( $contact_id, $args['update_data'] );

		}

		if ( ! empty( $args['apply_tags'] ) ) {
			// Apply the tags
			if ( wpf_get_contact_id() === $contact_id ) {

				wp_fusion()->user->apply_tags( $args['apply_tags'] );

			} elseif ( false !== $contact_id ) {

				wpf_log(
					'info',
					wpf_get_user_id( $contact_id ),
					'Applying abandoned cart tags for contact #' . $contact_id . ':',
					array(
						'tag_array' => $args['apply_tags'],
						'source'    => 'wpf-abandoned-cart',
					)
				);

				wp_fusion()->crm->apply_tags( $args['apply_tags'], $contact_id );

			}
		}

		do_action( 'wpf_abandoned_cart_created', $contact_id, $args );

	}

	/**
	 * Remove abandoned cart tags
	 *
	 * @access public
	 * @return void
	 */

	public function checkout_complete( $event ) {

		// The mepr-signup hook passes a transaction already.
		if ( is_a( $event, 'MeprTransaction' ) ) {
			$txn = $event;
		} else {
			$txn = $event->get_data();
		}

		$contact_id = wpf_get_contact_id( $txn->user_id, $txn->user_email );
		$user       = get_userdata( $txn->user_id );

		if ( wp_fusion_abandoned_cart()->carts->is_cart_pending( $contact_id, $user->user_email ) ) {
			wp_fusion_abandoned_cart()->carts->delete_cart( $contact_id, $user->user_email );
			return; // Cart hasn't been synced yet, don't need to do anything.
		}

		if ( $txn->subscription_id ) {

			// This is hacky but checking the transaction number seems to be the best way to check if the trial was
			// created. With Stripe it will look like seti_1N2qioH1J3FRLhcCsdvEJGkj.

			$subscription = new MeprSubscription( $txn->subscription_id );
			$is_trial     = ( $subscription->trial && false === strpos( $txn->trans_num, 'mp-txn' ) ) ? true : false;
		} else {
			$is_trial = false;
		}

		if ( 'active' === $txn->status || 'complete' === $txn->status || $is_trial ) {

			// Get tags to be removed.
			$remove_tags = wpf_get_option( 'abandoned_cart_apply_tags', array() );

			$settings = get_post_meta( $txn->product_id, 'wpf-settings-memberpress', true );

			if ( ! empty( $settings ) && ! empty( $settings['apply_tags_abandoned_cart'] ) ) {
				$remove_tags = array_merge( $remove_tags, $settings['apply_tags_abandoned_cart'] );
			}

			if ( ! empty( $remove_tags ) ) {

				wp_fusion()->user->remove_tags( $remove_tags, $txn->user_id );

			}
		}
	}


	/**
	 * Get cart recovery URL
	 *
	 * @access public
	 * @return string Recovery URL
	 */
	public function get_cart_recovery_url( $contact_id, $user_data ) {

		$membership_id = isset( $_POST['product_id'] ) ? intval( $_POST['product_id'] ) : intval( $_POST['mepr_product_id'] );

		$session = array(
			'user_data' => $user_data,
			'contents'  => $this->get_cart_items( $membership_id ),
			'discounts' => $this->get_cart_discounts( $user_data ),
			'source'    => 'memberpress',
		);

		wp_fusion_abandoned_cart()->carts->create_update_cart( $session, $contact_id );

		$url = add_query_arg( 'wpfrc', urlencode( $contact_id ), get_the_permalink( $membership_id ) );

		return $url;

	}

	/**
	 * Add settings to MemberPress meta box
	 *
	 * @access public
	 * @return mixed HTML output
	 */

	public function meta_box( $settings, $product ) {

		$defaults = array(
			'apply_tags_abandoned_cart' => array(),
		);

		$settings = array_merge( $defaults, $settings );

		echo '<br /><br /><label><strong>' . __( 'Apply Tags - Abandoned Cart', 'wp-fusion-abandoned-cart' ) . ':</strong></label><br />';

		$args = array(
			'setting'   => $settings['apply_tags_abandoned_cart'],
			'meta_name' => 'wpf-settings-memberpress',
			'field_id'  => 'apply_tags_abandoned_cart',
		);

		wpf_render_tag_multiselect( $args );

		echo '<br /><span class="description"><small>' . __( 'Apply these tags when someone begins to check out with this product. Tags will be removed after successful payment.', 'wp-fusion-abandoned-cart' ) . '</small></span>';

	}


}

new WPF_Abandoned_Cart_Memberpress();
