<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * Paid Memberships Pro integration class.
 *
 * @since 2.0.0
 */
class WPF_Abandoned_Cart_PMPro extends WPF_Abandoned_Cart_Integrations_Base {

	/**
	 * The slug for the integration.
	 *
	 * @since 2.0.0
	 * @var string
	 */
	public $slug = 'pmpro';

	/**
	 * Get things started.
	 *
	 * @since 2.0.0
	 */
	public function init() {

		add_action( 'pmpro_checkout_before_form', array( $this, 'checkout_begin' ) );
		add_action( 'pmpro_after_checkout', array( $this, 'checkout_complete' ), 10, 2 );

		// Admin settings.
		add_action( 'pmpro_membership_level_after_wp_fusion_settings', array( $this, 'membership_level_settings' ) );
	}

	/**
	 * Enqueue scripts on checkout page.
	 *
	 * @since 2.0.0
	 */
	public function should_enqueue_scripts() {

		if ( is_page( get_option( 'pmpro_checkout_page_id' ) ) && ! wpf_is_user_logged_in() ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Applies abandoned cart tags.
	 *
	 * @since 2.0.0
	 *
	 * @param bool|string $contact_id The contact ID, or false if unknown.
	 * @param array       $user_data  POST data from checkout.
	 */
	public function checkout_begin( $contact_id = false, $user_data = array() ) {

		// Don't run on initial checkout load for guests
		if ( ! wpf_is_user_logged_in() && empty( $contact_id ) ) {
			return;
		}

		if ( empty( $user_data ) && wpf_is_user_logged_in() ) {

			$user                    = wp_get_current_user();
			$user_data['user_email'] = $user->user_email;
			$user_data['first_name'] = $user->first_name;
			$user_data['last_name']  = $user->last_name;

		}

		if ( false === $contact_id && wpf_is_user_logged_in() ) {
			$contact_id = wpf_get_contact_id();

			if ( false === $contact_id ) {
				$contact_id = wpf_get_contact_id( wpf_get_current_user_id(), true ); // force a lookup in the CRM.
			}
		}

		if ( false === $contact_id ) {
			return; // Can't do anything at this point without a contact record.
		}

		$apply_tags = wpf_get_option( 'abandoned_cart_apply_tags', array() );

		$membership_id = $_POST['pmpro_level'] ?? intval( $_POST['pmpro_level'] );

		$settings = get_option( 'wpf_pmp_' . $membership_id, array() );

		if ( ! empty( $settings ) && ! empty( $settings['apply_tags_abandoned_cart'] ) ) {
			$apply_tags = array_merge( $apply_tags, $settings['apply_tags_abandoned_cart'] );
		}

		$membership = new PMPro_Membership_Level( $membership_id );

		$items = array(
			array(
				'product_id'  => $membership_id,
				'name'        => $membership->name,
				'quantity'    => 1,
				'price'       => $membership->billing_amount,
				'total'       => $membership->billing_amount,
				'product_url' => add_query_arg( 'pmpro_level', $membership_id, pmpro_url( 'checkout' ) ),
			),
		);

		/**
		 * Allows filtering the abandoned cart tags before they're applied.
		 *
		 * @since 2.0.0
		 *
		 * @param  array  $apply_tags The tags to be applied.
		 * @param  string $contact_id The contact ID.
		 */
		$apply_tags = apply_filters( 'wpf_abandoned_cart_apply_tags', $apply_tags, $contact_id );

		$args = array(
			'source'       => $this->slug,
			'contact_id'   => $contact_id,
			'update_data'  => $user_data,
			'apply_tags'   => $apply_tags,
			'cart_id'      => rand(),
			'recovery_url' => add_query_arg( 'pmpro_level', $membership_id, pmpro_url( 'checkout' ) ),
			'items'        => $items,
			'user_email'   => $user_data['user_email'],
			'provider'     => 'Paid Memberships Pro',
			'update'       => false,
			'currency'     => get_option( 'pmpro_currency' ),
		);

		$delay = wpf_get_option( 'abandoned_cart_delay' );

		if ( ! $delay ) {
			$this->process_abandoned_cart( $contact_id, $args );
		} else {
			wp_fusion_abandoned_cart()->carts->create_update_cart( $args, $contact_id );
		}
	}

	/**
	 * Process abandoned cart.
	 *
	 * @since 2.0.0
	 *
	 * @param string $contact_id  The contact ID.
	 * @param array  $args        The cart args.
	 * @param bool   $sync_fields Whether to sync fields.
	 */
	public function process_abandoned_cart( $contact_id, $args, $sync_fields = true ) {

		if ( $sync_fields && ! empty( wp_fusion()->crm->map_meta_fields( $args['update_data'] ) ) ) {

			wpf_log(
				'info',
				wpf_get_user_id( $contact_id ),
				'Syncing abandoned cart data for contact #' . $contact_id . ':',
				array(
					'meta_array_nofilter' => $args['update_data'],
					'source'              => 'wpf-abandoned-cart',
				)
			);

			wp_fusion()->crm->update_contact( $contact_id, $args['update_data'] );
		}

		if ( ! empty( $args['apply_tags'] ) ) {

			// Apply the tags.
			if ( wpf_get_contact_id() === $contact_id ) {

				wp_fusion()->user->apply_tags( $args['apply_tags'] );

			} elseif ( false !== $contact_id ) {

				wpf_log(
					'info',
					wpf_get_user_id( $contact_id ),
					'Applying abandoned cart tags for contact #' . $contact_id . ':',
					array(
						'tag_array' => $args['apply_tags'],
						'source'    => 'wpf-abandoned-cart',
					)
				);

				wp_fusion()->crm->apply_tags( $args['apply_tags'], $contact_id );

			}
		}

		do_action( 'wpf_abandoned_cart_created', $contact_id, $args );
	}

	/**
	 * Remove abandoned cart tags after checkout complete.
	 *
	 * @since 2.0.0
	 *
	 * @param int         $user_id The user ID.
	 * @param MemberOrder $order   The order object.
	 */

	public function checkout_complete( $user_id, $order ) {

		$contact_id = wpf_get_contact_id( $user_id );

		if ( wp_fusion_abandoned_cart()->carts->is_cart_pending( $contact_id, $order->{'Email'} ) ) {
			wp_fusion_abandoned_cart()->carts->delete_cart( $contact_id, $order->{'Email'} );
			return; // Cart hasn't been synced yet, don't need to do anything.
		}

		if ( 'success' === $order->status ) {

			$membership_level = $order->getMembershipLevel();

			$remove_tags = wpf_get_option( 'abandoned_cart_apply_tags', array() );
			$settings    = get_option( 'wpf_pmp_' . $membership_level->id );

			if ( ! empty( $settings ) && ! empty( $settings['apply_tags_abandoned_cart'] ) ) {
				$remove_tags = array_merge( $remove_tags, $settings['apply_tags_abandoned_cart'] );
			}

			if ( ! empty( $remove_tags ) ) {
				wp_fusion()->user->remove_tags( $remove_tags, $user_id );
			}
		}
	}

	/**
	 * Adds options to PMP membership level settings.
	 *
	 * @since 2.0.0
	 *
	 * @param PMPro_Membership_Level $level The membership level.
	 */
	public function membership_level_settings( $level ) {

		$settings = array(
			'apply_tags_abandoned_cart' => array(),
		);

		$settings = wp_parse_args( get_option( 'wpf_pmp_' . $level->id ), $settings );

		?>

		<table class="form-table" id="wp_fusion_tab">
			<tbody>
			<tr>
				<th scope="row" valign="top"><label><?php echo __( 'Apply Tags - Abandoned Cart', 'wp-fusion-abandoned-cart' ); ?>:</label></th>
				<td>
				<?php
					wpf_render_tag_multiselect(
						array(
							'setting'   => $settings['apply_tags_abandoned_cart'],
							'meta_name' => 'wpf-settings',
							'field_id'  => 'apply_tags_abandoned_cart',
						)
					);
				?>
					<br/>
					<small><?php _e( 'Apply these tags when someone begins to check out with this membership. Tags will be removed after successful enrollment.', 'wp-fusion-abandoned-cart' ); ?></small>
				</td>
			</tr>
			</tbody>
		</table>
		<?php
	}
}

new WPF_Abandoned_Cart_PMPro();
