<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

class WPF_Abandoned_Cart_Woocommerce extends WPF_Abandoned_Cart_Integrations_Base {

	/**
	 * The slug for the integration.
	 *
	 * @since 1.9.0
	 * @var string
	 */
	public $slug = 'woocommerce';

	/**
	 * Get things started.
	 *
	 * @since 1.0.0
	 */
	public function init() {

		add_action( 'wp_ajax_nopriv_wpf_progressive_update_cart', array( $this, 'progressive_update' ) );

		add_filter( 'wpf_configure_sections', array( $this, 'configure_sections' ) );
		add_filter( 'wpf_configure_settings', array( $this, 'register_settings' ), 20 );

		// Product settings.
		add_action( 'wpf_woocommerce_panel', array( $this, 'panel_content' ) );

		// Variations fields.
		add_action( 'woocommerce_product_after_variable_attributes', array( $this, 'variable_fields' ), 15, 3 );

		// Abandoned cart tracking.

		if ( wpf_get_option( 'abandoned_cart_add_to_cart' ) ) {

			// If we're syncing the cart with each update.

			add_action( 'woocommerce_add_to_cart', array( $this, 'cart_updated' ) );
			add_action( 'woocommerce_after_cart_item_quantity_update', array( $this, 'cart_updated' ) );
			add_action( 'woocommerce_cart_item_removed', array( $this, 'cart_updated' ) );

			add_action( 'wpf_user_created', array( $this, 'login' ) );
			add_action( 'wp_login', array( $this, 'login' ) );

		} else {

			// vs if we're just syncing it at checkout.
			add_action( 'woocommerce_before_checkout_form', array( $this, 'before_checkout_form' ) );
			add_action( 'woocommerce_blocks_checkout_enqueue_data', array( $this, 'before_checkout_form' ) );

		}

		// Cart recovery URL.
		add_action( 'init', array( $this, 'set_cart_recovery_session' ) );
		add_action( 'woocommerce_init', array( $this, 'recover_cart' ) );

		// After checkout complete.
		add_action( 'wpf_woocommerce_payment_complete', array( $this, 'checkout_complete' ), 20, 2 ); // 20 so we don't delete the saved cart before the Ecom addon runs
	}

	/**
	 * Enqueue scripts on checkout page.
	 *
	 * @since 2.0.0
	 * @return bool Whether to enqueue scripts.
	 */
	public function should_enqueue_scripts() {

		// We're not using wpf_is_user_logged_in() here so that the script is still enqueued during an auto login session.

		if ( ! is_user_logged_in() && ( is_checkout() || WC_Blocks_Utils::has_block_in_page( get_the_ID(), 'woocommerce/checkout' ) ) ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Fix namings for new woo blocks checkout (in the new checkout Woo only sends shipping fields if billing is the same).
	 *
	 * @since 2.0.0
	 *
	 * @param array $data The POSTed form data.
	 * @return array The formatted data.
	 */
	public function format_posted_data( $data ) {

		if ( isset( $data['data'] ) ) {

			// Classic checkout formatted data.

			parse_str( $data['data'], $form_data );

			$data = array_merge( $data, $form_data );

			if ( empty( $data['first_name'] ) ) {
				$data['first_name'] = $this->get_field_from_data( $data, 'first_name' );
			}

			if ( empty( $data['last_name'] ) ) {
				$data['last_name'] = $this->get_field_from_data( $data, 'last_name' );
			}

			if ( empty( $data['user_email'] ) ) {
				$data['user_email'] = $this->get_field_from_data( $data, 'email' );
			}

			unset( $data['data'] );

		}

		foreach ( $data as $key => $value ) {
			if ( strpos( $key, 'shipping' ) !== false || strpos( $key, 'billing' ) !== false ) {
				$fixed_key          = str_replace( '-', '_', $key );
				$data[ $fixed_key ] = $value;

				$billing_key = str_replace( 'shipping', 'billing', $fixed_key );
				if ( ! isset( $data[ $billing_key ] ) ) {
					$data[ $billing_key ] = $value;
				}
			}
		}

		return $data;
	}


	/**
	 * Progressively updates contact as additional fields are filled.
	 *
	 * @since 1.4.1
	 */
	public function progressive_update() {

		remove_action( 'wp_ajax_nopriv_wpf_progressive_update_cart', array( $this, 'progressive_update' ) );

		$post_data = array();
		parse_str( $_POST['data'], $post_data );

		$post_data['first_name'] = $this->get_field_from_data( $post_data, 'first_name' );
		$post_data['last_name']  = $this->get_field_from_data( $post_data, 'last_name' );
		$post_data['user_email'] = $this->get_field_from_data( $post_data, 'email' );

		if ( isset( $post_data['email_optin'] ) ) {
			unset( $post_data['email_optin'] );
		}

		$contact_id = sanitize_text_field( wp_unslash( $_POST['contact_id'] ) );

		wpf_log(
			'info',
			wpf_get_current_user_id(),
			'Abandoned cart updating contact #' . $contact_id . ' via progressive update.',
			array(
				'meta_array' => $post_data,
				'source'     => 'wpf-abandoned-cart',
			)
		);

		wp_fusion()->crm->update_contact( $contact_id, $post_data );

		wp_send_json_success();
	}



	/**
	 * Adds Addons tab if not already present.
	 *
	 * @return array Sections
	 */

	public function configure_sections( $page ) {

		if ( ! isset( $page['sections']['addons'] ) ) {
			$page['sections'] = wp_fusion()->settings->insert_setting_before( 'import', $page['sections'], array( 'addons' => __( 'Addons', 'wp-fusion-abandoned-cart' ) ) );
		}

		return $page;
	}


	/**
	 * Add fields to settings page.
	 *
	 * @access public
	 * @return array Settings
	 */

	public function register_settings( $settings ) {

		$settings['ec_woo_header'] = array(
			'title'   => __( 'Misc. Ecommerce Settings', 'wp-fusion-abandoned-cart' ),
			'type'    => 'heading',
			'section' => 'addons',
		);

		return $settings;
	}

	/**
	 * Display abandoned cart input on WPF / Woo panel.
	 *
	 * @return mixed HTML content.
	 */

	public function panel_content( $post_id ) {

		$settings = array( 'apply_tags_abandoned' => array() );

		$settings = wp_parse_args( get_post_meta( $post_id, 'wpf-settings-woo', true ), $settings );

		echo '<div class="options_group wpf-product">';

		echo '<p class="form-field"><label><strong>' . __( 'Abandoned Cart', 'wp-fusion-abandoned-cart' ) . '</strong></label></p>';

		echo '<p class="form-field"><label for="wpf-apply-tags-woo">' . __( 'Apply tags when cart abandoned', 'wp-fusion-abandoned-cart' ) . ':</label>';

		wpf_render_tag_multiselect(
			array(
				'setting'   => $settings['apply_tags_abandoned'],
				'meta_name' => 'wpf-settings-woo',
				'field_id'  => 'apply_tags_abandoned',
			)
		);

		echo '</p>';

		echo '</div>';
	}


	/**
	 * Adds tag multiselect to variation fields
	 *
	 * @access public
	 * @return mixed
	 */

	public function variable_fields( $loop, $variation_data, $variation ) {

		$defaults = array(
			'apply_tags_variation_abandoned' => array( $variation->ID => array() ),
		);

		$settings = wp_parse_args( get_post_meta( $variation->ID, 'wpf-settings-woo', true ), $defaults );

		echo '<div><p class="form-row form-row-full"><label for="wpf-apply-tags-woo">' . __( 'Apply tags when cart abandoned', 'wp-fusion-abandoned-cart' ) . ':</label>';

		wpf_render_tag_multiselect(
			array(
				'setting'   => $settings['apply_tags_variation_abandoned'][ $variation->ID ],
				'meta_name' => "wpf-settings-woo-variation[apply_tags_variation_abandoned][{$variation->ID}]",
			)
		);

		echo '</p></div>';
	}


	/**
	 * If a user registers or logs in and their cart already has contents, sync it.
	 *
	 * @param string|int $user_id The user login or user ID.
	 *
	 * @since 1.7.5
	 */
	public function login( $user_id ) {

		if ( ! is_numeric( $user_id ) ) {
			// wp_login passes a user_login.
			$user    = get_user_by( 'login', $user_id );
			$user_id = $user->ID;
		}

		if ( wpf_get_current_user_id() !== $user_id ) {
			return; // user was added by an admin or something.
		}

		if ( ! empty( WC()->cart ) && ! empty( WC()->cart->get_cart() ) && WC()->cart->get_cart_contents_count() && ! is_checkout() ) {

			$this->checkout_begin( wpf_get_contact_id( $user_id ) );

		}
	}

	/**
	 * Triggered at start of Woo checkout when the add-to-cart trigger is off.
	 *
	 * @since 1.0.0.
	 */
	public function before_checkout_form() {

		if ( is_admin() ) {
			return;
		}

		$this->checkout_begin();
	}

	/**
	 * When Trigger on Add to Cart is enabled, this watches for cart changes and triggers checkout_begin.
	 *
	 * @since 1.8.0.
	 */
	public function cart_updated() {

		$this->checkout_begin();
	}

	/**
	 * Get cart recovery URL
	 *
	 * @access public
	 * @return string Recovery URL
	 */

	public function get_cart_recovery_url( $contact_id, $user_data ) {

		$cart = array(
			'source'    => $this->slug,
			'contents'  => WC()->cart->get_cart(),
			'discounts' => $this->get_cart_discounts(),
			'user_data' => $user_data,
		);

		wp_fusion_abandoned_cart()->carts->create_update_cart( $cart, $contact_id );

		$destination = wpf_get_option( 'abandoned_cart_recovery_url_destination', 'checkout' );

		if ( 'checkout' === $destination ) {

			$url = wc_get_checkout_url();

		} elseif ( 'cart' === $destination ) {

			$url = wc_get_cart_url();

		} elseif ( 'current' === $destination ) {

			global $wp;

			if ( ! empty( $wp->request ) ) {

				$url = home_url( $wp->request );

			} elseif ( defined( 'DOING_AJAX' ) && DOING_AJAX ) {

					$url = $_SERVER['HTTP_REFERER'];

			} else {

				$url = wc_get_checkout_url();
			}
		}

		$url = add_query_arg( 'wpfrc', rawurlencode( $contact_id ), trailingslashit( $url ) );

		return $url;
	}

	/**
	 * Gets any abandoned cart tags to apply based on the current cart or order.
	 *
	 * @since 1.8.0
	 *
	 * @param WC_Order|bool $order The order.
	 * @return array The tags to apply.
	 */
	public function get_abandoned_cart_tags( $order = false ) {

		$apply_tags = wpf_get_option( 'abandoned_cart_apply_tags', array() );

		if ( $order ) {
			$items = $order->get_items();
		} else {
			$items = WC()->cart->get_cart();
		}

		foreach ( $items as $item ) {

			$product_id = $item['product_id'];
			$settings   = get_post_meta( $product_id, 'wpf-settings-woo', true );

			// Products.

			if ( ! empty( $settings ) && ! empty( $settings['apply_tags_abandoned'] ) ) {
				$apply_tags = array_merge( $apply_tags, $settings['apply_tags_abandoned'] );
			}

			// Variations.

			if ( ! empty( $item['variation_id'] ) ) {

				$variation_settings = get_post_meta( $item['variation_id'], 'wpf-settings-woo', true );

				if ( is_array( $variation_settings ) && isset( $variation_settings['apply_tags_variation_abandoned'][ $item['variation_id'] ] ) ) {
					$apply_tags = array_merge( $apply_tags, $variation_settings['apply_tags_variation_abandoned'][ $item['variation_id'] ] );
				}
			}
		}

		return $apply_tags;
	}

	/**
	 * Get cart ID from WooCommerce session
	 *
	 * @return bool / int Cart ID
	 */

	public function get_cart_id() {

		$session_key = WC()->session->get_customer_unique_id();

		global $wpdb;

		$table = $GLOBALS['wpdb']->prefix . 'woocommerce_sessions';

		$value = $wpdb->get_var( $wpdb->prepare( "SELECT session_id FROM $table WHERE session_key = %s", $session_key ) );

		return $value;
	}

	/**
	 * Gets information about the current customer.
	 *
	 * @since 1.8.0
	 *
	 * @param string $contact_id The contact ID.
	 * @return array The user data.
	 */
	public function get_customer_data( $contact_id ) {

		$user_data = array();

		if ( wpf_is_user_logged_in() ) {
			$user_id = wpf_get_current_user_id();
		} else {
			$user_id = wpf_get_user_id( $contact_id );
		}

		if ( $user_id ) {

			// If the user is registered, get their data from the database. This gets synced to the CRM and also stored in the recovery data.
			$user_data = wp_fusion()->user->get_user_meta( $user_id );

		} else {

			// Try to get it from the cart.

			$cart = wp_fusion_abandoned_cart()->carts->get_cart( $contact_id );

			if ( $cart ) {
				$user_data = $cart['user_data'];
			}
		}

		if ( empty( $user_data ) ) {

			// Finally, try to get it from the session.
			$user_data = WC()->session->get( 'customer' );

			if ( ! empty( $user_data ) ) {
				$user_data['user_email'] = $user_data['email'];
			}
		}

		// Clean up array stuff.
		foreach ( $user_data as $key => $value ) {

			if ( ! is_string( $value ) ) {
				unset( $user_data[ $key ] );
			}
		}

		return $user_data;
	}

	/**
	 * This is the main abandoned cart handler for WooCommerce. Syncs any custom fields,
	 * applies tags, and stores the cart recovery data.
	 *
	 * @since 1.0.0
	 *
	 * @param string|bool $contact_id    The contact ID.
	 * @param array       $user_data The customer data.
	 */
	public function checkout_begin( $contact_id = false, $user_data = array() ) {

		if ( empty( WC()->cart ) || empty( WC()->session ) ) {
			return; // just in case this gets triggered outside of WooCommerce somehow.
		}

		if ( false === $contact_id ) {

			$contact_id = $this->get_contact_id();

			if ( false === $contact_id ) {
				// Unknown contact ID, so we can't sync the cart.
				return;
			}
		}

		if ( empty( $user_data ) ) {
			$user_data = $this->get_customer_data( $contact_id );
		}

		//
		// Calculate and sync any enabled cart fields.
		//

		$update_data = array();

		if ( wpf_is_field_active( 'cart_recovery_url' ) || has_action( 'wpf_abandoned_cart_created' ) ) {
			// Only save the cart if the URL is enabled for sync, or the CRM supports cart recovery.
			$recovery_url = $this->get_cart_recovery_url( $contact_id, $user_data );
		} else {
			$recovery_url = false;
		}

		if ( ! WC()->session->get( 'wpf_recovery_url' ) ) {
			// Don't need to sync this twice if it's already in the session.
			$update_data['cart_recovery_url'] = $recovery_url;
		}

		// Cart value.

		WC()->cart->calculate_totals(); // Calc totals here so get_cart_contents_total() is accurate.
		$update_data['cart_value'] = WC()->cart->get_cart_contents_total();

		// Coupons.
		if ( WC()->cart->get_coupons() ) {

			$coupons = WC()->cart->get_coupons();
			$coupon  = reset( $coupons );

			if ( wpf_is_field_active( 'cart_discount_code' ) ) {
				$update_data['cart_discount_code'] = $coupon->get_code();
			}

			if ( wpf_is_field_active( 'cart_discount_amount' ) ) {
				$update_data['cart_discount_amount'] = WC()->cart->get_discount_total();
			}
		}

		// Product Names.
		if ( WC()->cart->get_cart_contents_count() > 0 ) {

			foreach ( WC()->cart->get_cart() as $item ) {
				if ( isset( $update_data['cart_product_name'] ) && ! empty( $update_data['cart_product_name'] ) ) {

					$update_data['cart_product_name'] .= ', ' . $item['data']->get_name();

				} else {
					$update_data['cart_product_name'] = $item['data']->get_name();
				}
			}
		}

		/**
		 * Allows filtering the abandoned cart data before it's synced to the CRM.
		 *
		 * @since 1.6.7
		 *
		 * @param  array  $update_data The data to be synced.
		 * @param  string $contact_id  The contact ID.
		 */
		$update_data = apply_filters( 'wpf_abandoned_cart_data', $update_data, $contact_id );

		/**
		 * Allows filtering the abandoned cart tags before they're applied.
		 *
		 * @since 1.6.7
		 *
		 * @param  array  $apply_tags The tags to be applied.
		 * @param  string $contact_id The contact ID.
		 */
		$apply_tags = apply_filters( 'wpf_abandoned_cart_apply_tags', $this->get_abandoned_cart_tags(), $contact_id );

		$args = array(
			'source'       => $this->slug,
			'contact_id'   => $contact_id,
			'user_data'    => $user_data,
			'update_data'  => $update_data,
			'apply_tags'   => $apply_tags,
			'cart_id'      => $this->get_cart_id(),
			'recovery_url' => $recovery_url,
			'items'        => $this->get_cart_contents(),
			'discounts'    => $this->get_cart_discounts(),
			'shipping'     => $this->get_cart_shipping(),
			'user_email'   => $user_data['user_email'],
			'provider'     => 'WooCommerce',
			'update'       => WC()->session->get( 'wpf_contact_id' ) ? true : false,
			'currency'     => get_woocommerce_currency(),
		);

		if ( ! wpf_get_option( 'abandoned_cart_delay' ) ) {
			$this->process_abandoned_cart( $contact_id, $args );
		} else {
			wp_fusion_abandoned_cart()->carts->create_update_cart( $args, $contact_id );
		}
	}

	/**
	 * Syncs the data and applies the tags for the cart, either in real time or after
	 * a delay, depending on the settings.
	 *
	 * @since 1.9.0
	 *
	 * @param string|bool $contact_id  The contact ID.
	 * @param array       $args        The cart data.
	 * @param bool        $sync_fields Whether to sync fields.
	 */
	public function process_abandoned_cart( $contact_id, $args, $sync_fields = true ) {

		// Create / update the contact.
		if ( $sync_fields && ! empty( wp_fusion()->crm->map_meta_fields( $args['update_data'] ) ) ) {

			wpf_log(
				'info',
				wpf_get_user_id( $contact_id ),
				'Syncing abandoned cart data for contact #' . $contact_id . ':',
				array(
					'meta_array_nofilter' => $args['update_data'],
					'source'              => 'wpf-abandoned-cart',
				)
			);

			wp_fusion()->crm->update_contact( $contact_id, $args['update_data'] );
		}

		// Apply the tags.

		if ( ! empty( $args['apply_tags'] ) ) {

			// Don't apply auto-discounts for abandoned carts.
			remove_action( 'wpf_tags_modified', array( wp_fusion()->integrations->woocommerce, 'maybe_apply_coupons' ) );

			// Apply the tags.

			if ( wpf_get_contact_id() === $contact_id ) {

				// If the user is logged in as the contact.

				wp_fusion()->user->apply_tags( $args['apply_tags'] );

			} elseif ( $contact_id ) {

				if ( null !== WC()->session ) {
					$session_tags = WC()->session->get( 'wpf_tags', array() );
				} else {
					$session_tags = array();
				}

				$diff = array_diff( (array) $args['apply_tags'], $session_tags );

				if ( ! empty( $diff ) ) {

					wpf_log(
						'info',
						wpf_get_user_id( $contact_id ),
						'Applying abandoned cart tags to contact #' . $contact_id . ':',
						array(
							'tag_array' => $diff,
							'source'    => 'wpf-abandoned-cart',
						)
					);

					wp_fusion()->crm->apply_tags( $diff, $contact_id );

					// Set the tags to the session storage so we don't need to reapply them every time.

					$session_tags = array_unique( array_merge( $session_tags, $args['apply_tags'] ) );

					if ( null !== WC()->session ) {
						WC()->session->set( 'wpf_tags', $session_tags );
					}
				}
			}
		}

		// Trigger enhanced ecommerce and other rich data (ActiveCampaign and Drip).

		do_action( 'wpf_abandoned_cart_created', $contact_id, $args );

		// Save the contact ID and recovery URL to the WooCommerce session.

		if ( null !== WC()->session ) {
			WC()->session->set( 'wpf_contact_id', $contact_id );
			WC()->session->set( 'wpf_recovery_url', $args['recovery_url'] );
		}
	}

	/**
	 * Get cart discounts.
	 *
	 * @return array
	 */
	public function get_cart_discounts() {

		$discounts = array();

		foreach ( WC()->cart->get_coupons() as $coupon ) {

			// Get the coupon code
			$coupon_code = $coupon->get_code();

			// Calculate the discount amount for this specific coupon
			$discount_amount = WC()->cart->get_coupon_discount_amount( $coupon_code );

			// Append the coupon and its corresponding discount amount to the array
			$discounts[] = array(
				'name'   => sanitize_text_field( $coupon_code ),
				'amount' => wc_format_decimal( $discount_amount ), // Format and sanitize the discount amount
			);

		}

		return $discounts;
	}

	/**
	 * Get cart shipping.
	 *
	 * @since 2.0.4
	 *
	 * @return array Shipping info.
	 */
	public function get_cart_shipping() {
		// Get the selected shipping methods from the session
		$chosen_shipping_methods = WC()->session->get( 'chosen_shipping_methods' );

		// Initialize variable to store the shipping method label
		$shipping_method_label = null;

		// Ensure we have at least one chosen shipping method
		if ( ! empty( $chosen_shipping_methods ) && is_array( $chosen_shipping_methods ) ) {
			$shipping_method_id = sanitize_text_field( $chosen_shipping_methods[0] );

			// Get shipping packages
			$packages = WC()->shipping()->get_packages();

			// Find the corresponding package for the selected shipping method
			foreach ( $packages as $package ) {
				foreach ( $package['rates'] as $rate ) {
					if ( $rate->id === $shipping_method_id ) {
						$shipping_method_label = $rate->get_label(); // Get human-readable label
						break 2; // Break out of both loops once the method is found
					}
				}
			}
		}

		$shipping = array(
			'total'  => wc_format_decimal( WC()->cart->get_shipping_total() ), // Get the shipping total
			'method' => $shipping_method_label,
		);

		return $shipping;
	}



	/**
	 * Get cart contents.
	 *
	 * @since 1.8.0
	 *
	 * @return array The contents.
	 */
	public function get_cart_contents() {

		$items = array();

		foreach ( WC()->cart->get_cart() as $item ) {

			$product_id = $item['product_id'];

			if ( empty( $product_id ) ) {

				// For variations get the setting from the parent.
				$product_id = $item['data']->get_parent_id();

			}

			$product = wc_get_product( $product_id );

			//
			// Put together the item data
			//

			$item_data = array(
				'product_id'  => (string) $product_id,
				'name'        => get_the_title( $product_id ), // Needs to be get_the_title() so it works with WP Multilang.
				'quantity'    => $item['quantity'],
				'total'       => isset( $item['line_subtotal'] ) ? round( $item['line_subtotal'], 2 ) : 0,
				'product_url' => get_the_permalink( $product_id ),
				'description' => $product->get_short_description(),
			);

			$size = wpf_get_option( 'abandoned_cart_image_size', 'medium' );

			$image = wp_get_attachment_image_src( get_post_thumbnail_id( $product_id ), $size );

			if ( ! empty( $image ) ) {
				$item_data['image_url'] = $image[0];
			}

			$item_data['price'] = (float) $item_data['total'] / (float) $item_data['quantity'];

			if ( wc_prices_include_tax() ) {

				$item_data['total'] += $item['line_subtotal_tax'];
				$item_data['price'] += round( $item['line_subtotal_tax'] / $item_data['quantity'], 2 );

			}

			if ( ! empty( get_post_meta( $product_id, '_sku', true ) ) ) {
				$item_data['sku'] = get_post_meta( $product_id, '_sku', true );
			}

			//
			// Categories
			//

			if ( 'attributes' === wpf_get_option( 'abandoned_cart_categories' ) ) {

				// Attributes as categories.

				$attributes = $item['data']->get_attributes();

				if ( ! empty( $attributes ) ) {

					foreach ( $attributes as $name => $value ) {

						if ( is_object( $value ) ) {
							continue; // Fixes Recoverable fatal error: Object of class WC_Product_Attribute could not be converted to string
						}

						$taxonomy = wc_attribute_taxonomy_name( str_replace( 'attribute_pa_', '', urldecode( $name ) ) );

						if ( taxonomy_exists( $taxonomy ) ) {

							// If this is a term slug, get the term's nice name.
							$term = get_term_by( 'slug', $value, $taxonomy );

							if ( ! is_wp_error( $term ) && $term && $term->name ) {
								$value = $term->name;
							}
							$label = wc_attribute_label( $taxonomy );

						} else {

							// If this is a custom option slug, get the options name.
							$value = apply_filters( 'woocommerce_variation_option_name', $value, null, $taxonomy, $item['data'] );
							$label = wc_attribute_label( str_replace( 'attribute_', '', $name ), $item['data'] );
						}

						$item_data['categories'][] = $label . ': ' . $value;

					}
				}
			} else {

				// Categories as categories

				$product_terms = get_the_terms( $product_id, 'product_cat' );

				if ( ! empty( $product_terms ) ) {

					$item_data['categories'] = array();

					foreach ( $product_terms as $term ) {

						$item_data['categories'][] = $term->name;

					}
				}
			}

			// Variations.

			if ( isset( $item['variation_id'] ) && $item['variation_id'] != 0 ) {

				// Data.

				$item_data['product_variant_id'] = $item['variation_id'];
				$item_data['name']               = get_the_title( $item['variation_id'] );

				$image = wp_get_attachment_image_src( get_post_thumbnail_id( $item['variation_id'] ), $size );

				if ( ! empty( $image ) ) {
					$item_data['image_url'] = $image[0];
				}
			}

			$items[] = $item_data;

		}

		return $items;
	}

	/**
	 * Detects a recovery URL and sets the session.
	 *
	 * @since 1.8.0
	 */
	public function set_cart_recovery_session() {

		if ( isset( $_GET['wpfrc'] ) ) {

			$contact_id = urldecode( $_GET['wpfrc'] );
			$contact_id = str_replace( ' ', '-', $contact_id ); // + symbols become spaces when URL encoded
			$contact_id = sanitize_text_field( $contact_id );

			// Some ESPs append query vars to the end of the URL (like UTM tracking links). This cleans those up.
			if ( false !== strpos( $contact_id, '?' ) ) {
				$contact_id = explode( '?', $contact_id );
				$contact_id = $contact_id[0];
			}

			// Save the contact ID to the WooCommerce session.
			WC()->session->set( 'wpf_contact_id', $contact_id );

		}
	}

	/**
	 * Gets the contact ID for abandoned cart tracking purposes.
	 *
	 * This doesn't check the $_GET parameter because we don't need to re-sync the cart
	 * immediately after a recovery URL is visited.
	 *
	 * @since 2.0.4
	 *
	 * @return string|bool Contact ID, or false if not found.
	 */
	public function get_contact_id() {

		if ( wpf_is_user_logged_in() ) {

			$contact_id = wpf_get_contact_id();

		} elseif ( null !== WC()->session && WC()->session->get( 'wpf_contact_id' ) ) {

			$contact_id = WC()->session->get( 'wpf_contact_id', false );

		} else {

			$contact_id = false;

		}

		return apply_filters( 'wpf_abandoned_cart_get_contact_id', $contact_id );
	}

	/**
	 * Recover cart URL processing
	 *
	 * @access public
	 * @return void
	 */

	public function recover_cart() {

		if ( is_admin() ) {
			return;
		}

		if ( ! empty( $_GET['wpfrc'] ) ) {

			$contact_id = sanitize_text_field( urldecode( $_GET['wpfrc'] ) );

			$cart = wp_fusion_abandoned_cart()->carts->get_cart( $contact_id );

			if ( ! empty( $cart ) ) {

				if ( isset( $cart['source'] ) && 'woocommerce' !== $cart['source'] ) {
					return; // don't recover carts from other sources.
				}

				if ( isset( $cart['user_data'] ) ) {

					// Pre-fill the checkout fields with the recovered data.

					foreach ( $cart['user_data'] as $key => $value ) {

						if ( 0 === strpos( $key, 'billing_' ) || 0 === strpos( $key, 'shipping_' ) ) {

							// Construct the method name.
							$method_name = 'set_' . $key;

							// Check if the method exists in the WC_Customer class.
							if ( WC()->customer && method_exists( WC()->customer, $method_name ) ) {
								// Call the method dynamically.
								call_user_func( array( WC()->customer, $method_name ), $value );
							}
						}
					}

					// Start tracking session.
					do_action( 'wpf_guest_contact_updated', $contact_id, $cart['user_data']['user_email'] );

				}

				// No need to send another abandoned cart.
				remove_action( 'woocommerce_add_to_cart', array( $this, 'add_to_cart' ) );
				remove_action( 'woocommerce_before_checkout_form', array( $this, 'before_checkout_form' ) );
				remove_action( 'woocommerce_blocks_checkout_enqueue_data', array( $this, 'before_checkout_form' ) );

				WC()->cart->set_cart_contents( $cart['contents'] );

				if ( ! empty( $cart['discounts'] ) ) {

					foreach ( $cart['discounts'] as $discount ) {
						WC()->cart->add_discount( $discount['name'] );
					}
				}

				WC()->session->set( 'wpf_contact_id', $contact_id );

			} else {
				wpf_log( 'notice', 0, 'A cart recovery link was visited for contact #' . $contact_id . ' but no cart was found in the database.' );
			}

			// Auto discounts.
			$contact_tags = wp_fusion()->crm->get_tags( $contact_id );

			if ( is_wp_error( $contact_tags ) ) {
				wpf_log( 'error', 0, 'Unable to load tags for contact #' . $contact_id . ': ' . $contact_tags->get_error_message() );
			}

			wp_fusion()->integrations->woocommerce->maybe_apply_coupons( null, $contact_tags );

		}
	}


	/**
	 * Remove abandoned cart tags
	 *
	 * @access public
	 * @return void
	 */

	public function checkout_complete( $order_id, $contact_id ) {

		$order = wc_get_order( $order_id );

		if ( wp_fusion_abandoned_cart()->carts->is_cart_pending( $contact_id, $order->get_billing_email() ) ) {
			wp_fusion_abandoned_cart()->carts->delete_cart( $contact_id, $order->get_billing_email() );
		}

		// Get tags to be removed.
		$remove_tags = $this->get_abandoned_cart_tags( $order );

		// Remove tags.
		if ( ! empty( $remove_tags ) ) {

			$user_id = $order->get_user_id();

			if ( ! empty( $user_id ) && wpf_has_tag( $remove_tags, $user_id ) ) {

				// We'll check for the tags on the user record in case the user just
				// registered and the tags haven't loaded from the CRM yet.

				wp_fusion()->user->remove_tags( $remove_tags, $user_id );

			} else {

				wpf_log(
					'info',
					0,
					'Removing abandoned cart tags from contact #' . $contact_id . ':',
					array(
						'tag_array' => $remove_tags,
						'source'    => 'wpf-abandoned-cart',
					)
				);

				wp_fusion()->crm->remove_tags( $remove_tags, $contact_id );

			}
		}

		//
		// Reset cart fields.
		//

		$update_data = array();

		if ( wpf_is_field_active( 'cart_recovery_url' ) ) {
			$update_data['cart_recovery_url'] = null;
		}

		if ( wpf_is_field_active( 'cart_value' ) ) {
			$update_data['cart_value'] = null;
		}

		if ( ! empty( $update_data ) ) {

			wpf_log(
				'info',
				0,
				'Clearing out abandoned cart fields for contact #' . $contact_id . ':',
				array(
					'meta_array_nofilter' => $update_data,
					'source'              => 'wpf-abandoned-cart',
				)
			);

			wp_fusion()->crm->update_contact( $contact_id, $update_data );
		}

		// Clear out saved cart.

		wp_fusion_abandoned_cart()->carts->delete_cart( $contact_id );

		do_action( 'wpf_abandoned_cart_recovered', $contact_id, $order_id );
	}
}

new WPF_Abandoned_Cart_Woocommerce();
