<?php

/**
 * Groundhogg REST CRM integration.
 *
 * @since 1.25.0
 */
class WPF_EC_Groundhogg_REST {

	/**
	 * Lets other integrations know which features are supported by the CRM.
	 */
	public $supports = array( 'deal_stages' );


	/**
	 * Get things started.
	 *
	 * @since 1.25.0
	 */
	public function init() {

		add_filter( 'wpf_configure_settings', array( $this, 'register_settings' ), 15, 2 );

		add_action( 'wpf_sync', array( $this, 'sync' ) );

		// Sync data on first run.
		$pipelines = wpf_get_option( 'groundhogg-rest_pipelines' );

		if ( ! is_array( $pipelines ) ) {
			$this->sync();
		}

	}


	/**
	 * Add fields to settings page.
	 *
	 * @since 1.25.0
	 *
	 * @return array Settings
	 */
	public function register_settings( $settings, $options ) {

		$settings['ec_groundhogg_rest_header'] = array(
			'title'   => __( 'Groundhogg REST Ecommerce Settings', 'wp-fusion' ),
			'type'    => 'heading',
			'section' => 'ecommerce',
		);

		$settings['groundhogg_rest_pipeline_stage'] = array(
			'title'       => __( 'Deal Stage', 'wp-fusion' ),
			'type'        => 'select',
			'section'     => 'ecommerce',
			'placeholder' => __( 'Select a Stage', 'wp-fusion' ),
			'choices'     => isset( $options['groundhogg-rest_pipelines'] ) ? $options['groundhogg-rest_pipelines'] : array(),
			'desc'        => __( 'Select a default pipeline and stage for new deals.', 'wp-fusion' ),
		);

		return $settings;

	}


	/**
	 * Syncs pipelines on plugin install or when Resynchronize is clicked.
	 *
	 * @since 1.25.0
	 */
	public function sync() {
		$request  = wp_fusion()->crm->url . '/pipelines';
		$response = wp_safe_remote_get( $request, wp_fusion()->crm->get_params() );

		if ( is_wp_error( $response ) ) {
			return $response;
		}

		$pipelines = json_decode( wp_remote_retrieve_body( $response ) );

		$statuses = array(
			'in_progress' => 'Active',
			'won'         => 'Won',
			'lost'        => 'Lost',
		);

		$final_pipelines = array();

		if ( ! empty( $pipelines->items ) ) {
			foreach ( $pipelines->items as $pipeline ) {

				foreach ( $pipeline->stages as $stage ) {
					foreach ( $statuses as $status_id => $status ) {
						$final_pipelines[ $pipeline->ID . '+' . $stage->ID . '+' . $status_id ] = $pipeline->data->title . ' &raquo; ' . $stage->data->stage_title . ' &raquo; ' . $status;
					}
				}
			}
		}

		wp_fusion()->settings->set( 'groundhogg-rest_pipelines', $final_pipelines );

	}

	/**
	 * Add an order.
	 *
	 * @since 1.25.0
	 *
	 * @param int    $order_id   The order ID in WordPress.
	 * @param string $contact_id The contact ID in the CRM.
	 * @param array  $order_args The order details.
	 * @return int|WP_Error The deal ID or error.
	 */
	public function add_order( $order_id, $contact_id, $order_args ) {

		if ( empty( wpf_get_option( 'groundhogg_rest_pipeline_stage' ) ) ) {
			wpf_log( 'notice', 0, 'To sync orders with groundhogg rest you must first select a deal stage from the Enhanced Ecommerce tab in the WP Fusion settings.' );
			return false;
		}

		if ( ! empty( $order_args['deal_stage'] ) ) {
			$pipeline_stage = $order_args['deal_stage'];
		} else {
			$pipeline_stage = wpf_get_option( 'groundhogg_rest_pipeline_stage' );
		}

		$pipeline_stage    = explode( '+', $pipeline_stage );
		$pipeline_id       = $pipeline_stage[0];
		$pipeline_stage_id = $pipeline_stage[1];

		if ( isset( $pipeline_stage[2] ) ) {
			$status = $pipeline_stage[2];
		} else {
			$status = 'in_progress';
		}

		$product_interest = array();

		foreach ( $order_args['products'] as $product ) {

			$product_interest[] = array(
				$product['name'],
				$product['price'],
				$product['qty'],
			);

		}

		$date = gmdate( 'Y-m-d', $order_args['order_date'] );

		$data = array(
			'title'                => $order_args['order_label'],
			'status'               => $status,
			'deal_value'           => floatval( $order_args['total'] ),
			'contact_id'           => $contact_id,
			'stage_id'             => $pipeline_stage_id,
			'pipeline_id'          => $pipeline_id,
			'projected_close_date' => $date,
		);

		/**
		 * Filters the deal data.
		 *
		 * @since 1.25.0
		 *
		 * @param array $data     The deal data.
		 * @param int   $order_id ID of the order.
		 */

		$data = apply_filters( 'wpf_ecommerce_groundhogg_add_deal', $data, $order_id );

		wpf_log(
			'info',
			$order_args['user_id'],
			'Adding <a href="' . $order_args['order_edit_link'] . '" target="_blank">' . $order_args['order_label'] . '</a>:',
			array(
				'meta_array_nofilter' => $data,
				'source'              => 'wpf-ecommerce',
			)
		);

		$request = wp_fusion()->crm->url . '/deals';
		$params  = wp_fusion()->crm->get_params();

		$params['body'] = wp_json_encode(
			array(
				'data' => $data,
				'meta' => array(
					'product_interest' => $product_interest,
				),
			),
		);

		$response = wp_safe_remote_post( $request, $params );

		if ( is_wp_error( $response ) ) {
			return $response;
		}

		$deal = json_decode( wp_remote_retrieve_body( $response ) );

		$deal_id = $deal->item->ID;

		// Link the deal to the contact.

		$data = array(
			'other_id'   => $contact_id,
			'other_type' => 'contact',
		);

		$request       .= '/' . $deal_id . '/relationships';
		$params['body'] = wp_json_encode( $data );

		$response = wp_safe_remote_post( $request, $params );

		if ( is_wp_error( $response ) ) {
			return $response;
		}

		return $deal_id;

	}

	/**
	 * Update a deal stage when an order status is changed.
	 *
	 * @since 1.25.0
	 *
	 * @param int    $deal_id  The deal ID.
	 * @param string $stage    The deal stage.
	 * @param int    $order_id The order ID.
	 * @return bool|WP_Error True on success, error on fail.
	 */
	public function change_stage( $deal_id, $stage, $order_id ) {

		$pipeline_stage = explode( '+', $stage );
		$pipeline_id    = $pipeline_stage[0];
		$stage_id       = $pipeline_stage[1];

		if ( isset( $pipeline_stage[2] ) ) {
			$data['status'] = $pipeline_stage[2];
		}

		$request = wp_fusion()->crm->url . '/deals/' . $deal_id;
		$params  = wp_fusion()->crm->get_params();

		$params['body'] = wp_json_encode(
			array(
				'data' => array(
					'stage_id'    => $stage_id,
					'pipeline_id' => $pipeline_id,
				),
			),
		);

		$response = wp_safe_remote_post( $request, $params );

		if ( is_wp_error( $response ) ) {
			return $response;
		}

		return true;

	}

}
