<?php

/**
 * Groundhogg CRM integration.
 *
 * @since 1.25.0
 */
class WPF_EC_Groundhogg {

	/**
	 * Lets other integrations know which features are supported by the CRM.
	 */
	public $supports = array( 'deal_stages' );

	/**
	 * Get things started.
	 *
	 * @since 1.25.0
	 */
	public function init() {

		add_filter( 'wpf_configure_settings', array( $this, 'register_settings' ), 15, 2 );

		add_action( 'wpf_sync', array( $this, 'sync' ) );

		// Sync data on first run.
		$pipelines = wpf_get_option( 'groundhogg_pipelines' );

		if ( ! is_array( $pipelines ) ) {
			$this->sync();
		}

	}


	/**
	 * Add fields to settings page.
	 *
	 * @since 1.25.0
	 *
	 * @return array Settings
	 */
	public function register_settings( $settings, $options ) {

		$settings['ec_groundhogg_header'] = array(
			'title'   => __( 'Groundhogg Ecommerce Settings', 'wp-fusion' ),
			'type'    => 'heading',
			'section' => 'ecommerce',
		);

		$settings['groundhogg_pipeline_stage'] = array(
			'title'       => __( 'Deal Stage', 'wp-fusion' ),
			'type'        => 'select',
			'section'     => 'ecommerce',
			'placeholder' => __( 'Select a Stage', 'wp-fusion' ),
			'choices'     => isset( $options['groundhogg_pipelines'] ) ? $options['groundhogg_pipelines'] : array(),
			'desc'        => __( 'Select a default pipeline and stage for new deals.', 'wp-fusion' ),
		);

		return $settings;

	}


	/**
	 * Syncs pipelines on plugin install or when Resynchronize is clicked.
	 *
	 * @since 1.25.0
	 */
	public function sync() {

		$pipelines       = \Groundhogg\Plugin::$instance->dbs->get_db( 'pipelines' )->query();
		$pipeline_stages = \Groundhogg\Plugin::$instance->dbs->get_db( 'pipeline_stages' )->query();

		$statuses = array(
			'in_progress' => 'Active',
			'won'         => 'Won',
			'lost'        => 'Lost',
		);

		$final_pipelines = array();

		if ( ! empty( $pipelines ) ) {
			foreach ( $pipelines as $pipeline ) {

				foreach ( $pipeline_stages as $stage ) {
					if ( $stage->pipeline_id !== $pipeline->ID ) {
						continue;
					}
					foreach ( $statuses as $status_id => $status ) {
						$final_pipelines[ $pipeline->ID . '+' . $stage->ID . '+' . $status_id ] = $pipeline->title . ' &raquo; ' . $stage->stage_title . ' &raquo; ' . $status;
					}
				}
			}
		}

		wp_fusion()->settings->set( 'groundhogg_pipelines', $final_pipelines );

	}

	/**
	 * Add an order.
	 *
	 * @since 1.25.0
	 *
	 * @param int    $order_id   The order ID in WordPress.
	 * @param string $contact_id The contact ID in the CRM.
	 * @param array  $order_args The order details.
	 * @return int|false|WP_Error The deal ID or error.
	 */
	public function add_order( $order_id, $contact_id, $order_args ) {

		if ( empty( wpf_get_option( 'groundhogg_pipeline_stage' ) ) ) {
			wpf_log( 'notice', 0, 'To sync orders with groundhogg you must first select a deal stage from the Enhanced Ecommerce tab in the WP Fusion settings.' );
			return false;
		}

		if ( ! empty( $order_args['deal_stage'] ) ) {
			$pipeline_stage = $order_args['deal_stage'];
		} else {
			$pipeline_stage = wpf_get_option( 'groundhogg_pipeline_stage' );
		}

		$pipeline_stage    = explode( '+', $pipeline_stage );
		$pipeline_id       = $pipeline_stage[0];
		$pipeline_stage_id = $pipeline_stage[1];

		if ( isset( $pipeline_stage[2] ) ) {
			$status = $pipeline_stage[2];
		} else {
			$status = 'in_progress';
		}

		$product_interest = array();

		foreach ( $order_args['products'] as $product ) {

			$product_interest[] = array(
				$product['name'],
				$product['price'],
				$product['qty'],
			);

		}

		$date = gmdate( 'Y-m-d', $order_args['order_date'] );

		$data = array(
			'title'                => $order_args['order_label'],
			'status'               => $status,
			'deal_value'           => floatval( $order_args['total'] ),
			'contact_id'           => $contact_id,
			'stage_id'             => $pipeline_stage_id,
			'pipeline_id'          => $pipeline_id,
			'projected_close_date' => $date,
		);

		/**
		 * Filters the deal data.
		 *
		 * @since 1.25.0
		 *
		 * @param array $data     The deal data.
		 * @param int   $order_id ID of the order.
		 */

		$data = apply_filters( 'wpf_ecommerce_groundhogg_add_deal', $data, $order_id );

		wpf_log(
			'info',
			$order_args['user_id'],
			'Adding <a href="' . $order_args['order_edit_link'] . '" target="_blank">' . $order_args['order_label'] . '</a>:',
			array(
				'meta_array_nofilter' => $data,
				'source'              => 'wpf-ecommerce',
			)
		);

		if ( ! class_exists( 'GroundhoggPipeline\Plugin' ) ) {
			wpf_log( 'error', 0, __( 'Groundhogg Pipeline addons has to be activated.', 'wpf-ecommerce' ) );
			return false;
		}

		$deal = new \GroundhoggPipeline\Classes\Deal();

		$deal->create( $data );

		if ( ! $deal->exists() ) {
			return false;
		}

		$deal->update_meta( 'product_interest', $product_interest );

		$contact = new \Groundhogg\Contact( $contact_id );

		$deal->add_note( wpautop( __( 'Adding <a href="' . $order_args['order_edit_link'] . '" target="_blank">' . $order_args['order_label'] . '</a>, By WP Fusion', 'wpf-ecommerce' ) ) );

		$deal->create_relationship( $contact );

		return $deal->ID;

	}

	/**
	 * Update a deal stage when an order status is changed.
	 *
	 * @since 1.25.0
	 *
	 * @param int    $deal_id  The deal ID.
	 * @param string $stage    The deal stage.
	 * @param int    $order_id The order ID.
	 * @return bool|WP_Error True on success, error on fail.
	 */
	public function change_stage( $deal_id, $stage, $order_id ) {

		$pipeline_stage = explode( '+', $stage );
		$pipeline_id    = $pipeline_stage[0];
		$stage_id       = $pipeline_stage[1];

		if ( isset( $pipeline_stage[2] ) ) {
			$data['status'] = $pipeline_stage[2];
		}

		$deal = new \GroundhoggPipeline\Classes\Deal( $deal_id );
		if ( empty( $deal ) || is_wp_error( $deal ) ) {
			return;
		}

		if ( $deal->get_pipeline_id() == $pipeline_id ) {
			$deal->move( $stage_id );
		}

		return true;

	}

}
