<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

use SureCart\Integrations\Contracts\IntegrationInterface;
use SureCart\Integrations\Contracts\PurchaseSyncInterface;
use SureCart\Integrations\IntegrationService;

/**
 * SureCart integration.
 *
 * @since 1.25.0
 */

class WPF_EC_SureCart_Integration extends IntegrationService implements IntegrationInterface, PurchaseSyncInterface {
	/**
	 * Get the slug for the integration.
	 *
	 * @return string
	 */
	public function getName() {
		return 'wp-fusion/ecommerce';
	}

	/**
	 * Get the SureCart model used for the integration.
	 * Only 'product' is supported at this time.
	 *
	 * @return string
	 */
	public function getModel() {
		return 'product';
	}

	/**
	 * Get the integration logo url.
	 * This can be to a png, jpg, or svg for example.
	 *
	 * @return string
	 */
	public function getLogo() {
		return esc_url( WPF_DIR_URL . '/assets/img/logo-sm-trans.png' );
	}

	/**
	 * The display name for the integration in the dropdown.
	 *
	 * @return string
	 */
	public function getLabel() {
		return sprintf( __( '%1$s Product', 'wp-fusion' ), wp_fusion()->crm->name );
	}

	/**
	 * The label for the integration item that will be chosen.
	 *
	 * @return string
	 */
	public function getItemLabel() {
		return sprintf( __( '%1$s Product', 'wp-fusion' ), wp_fusion()->crm->name );
	}

	/**
	 * Help text for the integration item chooser.
	 *
	 * @return string
	 */
	public function getItemHelp() {
		return sprintf( __( '%1$s Product for WP Fusion Ecommerce', 'wp-fusion' ), wp_fusion()->crm->name );
	}

	/**
	 * Get item listing for the integration.
	 * These are a list of item the merchant can choose from when adding an integration.
	 *
	 * @param array  $items The integration items.
	 * @param string $search The search term.
	 *
	 * @return array The items for the integration.
	 */
	public function getItems( $items = array(), $search = '' ) {

		$final_products     = array();
		$available_products = get_option( 'wpf_' . wp_fusion()->crm->slug . '_products', array() );
		foreach ( $available_products as $key => $value ) {

			if ( ! empty( $search ) ) {
				if ( false === strpos( strtolower( $value ), strtolower( $search ) ) ) {
					continue;
				}
			}

			$final_products[] = array(
				'id'    => $key,
				'label' => $value,
			);
		}
		return $final_products;
	}

	/**
	 * Get the individual item.
	 *
	 * @param string $role The item role.
	 *
	 * @return array The item for the integration.
	 */
	public function getItem( $product ) {

		$product = urldecode( $product );

		$available_products = get_option( 'wpf_' . wp_fusion()->crm->slug . '_products', array() );

		if ( ! isset( $available_products[ $product ] ) ) {
			return false;
		}

		return array(
			'id'    => $product,
			'label' => isset( $available_products[ $product ] ) ? $available_products[ $product ] : sprintf( __( 'Unknown Product: %s', 'wp-fusion' ), $product ),
		);
	}

	/**
	 * Send order data to CRM when the purchase is created.
	 *
	 * @param \SureCart\Models\Integration $integration The integration.
	 * @param \WP_User                     $user        The user.
	 *
	 * @return bool Returns true if the tag is applied.
	 */
	public function onPurchaseCreated( $integration, $user ) {

		// This runs per product integration, not per purchase.
		return true;
	}

	/**
	 * Send order data to CRM when the purchase is invoked
	 *
	 * @param \SureCart\Models\Integration $integration The integration.
	 * @param \WP_User                     $user        The user.
	 *
	 * @return bool Returns true if the tag is applied.
	 */
	public function onPurchaseInvoked( $integration, $user ) {

		// This runs per product integration, not per purchase.
		return true;
	}

	/**
	 * Refund Order when the purchase is revoked.
	 *
	 * @param \SureCart\Models\Integration $integration The integration.
	 * @param \WP_User                     $user        The user.
	 *
	 * @return bool Returns true if the order is refunded.
	 */
	public function onPurchaseRevoked( $integration, $user ) {

		// This runs per product integration, not per purchase.
		return true;

	}

}