<?php

use SureCart\Models\Integration;
use SureCart\Models\Purchase;
use SureCart\Models\Checkout;
use SureCart\Models\Charge;
use SureCart\Models\PaymentIntent;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * SureCart integration.
 *
 * @since 1.25.0
 *
 * @link https://wpfusion.com/documentation/ecommerce/surecart/#enhanced-ecommerce
 */
class WPF_EC_SureCart extends WPF_EC_Integrations_Base {

	/**
	 * The integration slug.
	 *
	 * @since 1.25.0
	 * @var string Slug.
	 */
	public $slug = 'surecart';

	/**
	 * Get things started.
	 *
	 * @since 1.25.0
	 */
	public function init() {

		add_action( 'surecart/checkout_confirmed', array( $this, 'send_order_data' ) );
		add_action( 'surecart/subscription_renewed', array( $this, 'renew_subscription' ) );
		add_action( 'surecart/models/refund/created', array( $this, 'refund_product' ) );

		if ( in_array( 'products', wp_fusion_ecommerce()->crm->supports ) ) {

			// Link products between SureCart and the CRM.

			$this->includes();
			( new WPF_EC_SureCart_Integration() )->bootstrap();
		}

		// Export functions.
		add_filter( 'wpf_export_options', array( $this, 'export_options' ) );
		add_filter( 'wpf_batch_surecart_ecom_init', array( $this, 'batch_init' ) );
		add_action( 'wpf_batch_surecart_ecom', array( $this, 'batch_step' ) );
	}

	/**
	 * Refund product.
	 *
	 * @since 1.25.1
	 *
	 * @param SureCart\Models\Refund $refund The refund.
	 */
	public function refund_product( $refund ) {
		if ( strtolower( $refund->status ) !== 'succeeded' ) {
			return;
		}

		$charge = Charge::with(
			array(
				'checkout',
				'checkout.customer',
				'checkout.payment_intent',
				'checkout.line_items',
				'line_item.price',
				'price.product',
				'line_items.variant',
			)
		)->find( $refund->charge );

		$refund_text = __( 'refunded', 'wp-fusion' );
		if ( $charge->fully_refunded === false ) {
			$refund_text = __( 'partially refunded', 'wp-fusion' );
		}

		$checkout = $charge->checkout;

		$invoice_id = $checkout->metadata->{ 'wpf_ec_' . wp_fusion()->crm->slug . '_invoice_id' };

		if ( empty( $invoice_id ) ) {
			return false;
		}

		$contact_id = wp_fusion()->crm->get_contact_id( $checkout->email );
		$order_args = wp_fusion_ecommerce()->integrations->surecart->get_order_args( $checkout );

		$result = wp_fusion_ecommerce()->crm->refund_order( $invoice_id, $this->format_price( $checkout->net_paid_amount ), $this->format_price( $refund->amount ), $order_args, $contact_id );

		if ( is_wp_error( $result ) ) {

			wpf_log( $result->get_error_code(), $order_args['user_id'], 'Error marking SureCart order marked as <a href="' . admin_url( 'admin.php?page=sc-orders&action=edit&id=' . $charge->checkout->order ) . '" target="_blank">#' . $charge->checkout->order . '</a> ' . $refund_text . ': ' . $result->get_error_message(), array( 'source' => 'wpf-ecommerce' ) );
			return false;
		}

		wpf_log( 'info', $order_args['user_id'], 'SureCart order marked as <a href="' . admin_url( 'admin.php?page=sc-orders&action=edit&id=' . $charge->checkout->order ) . '" target="_blank">#' . $charge->checkout->order . '</a> ' . $refund_text . ' in ' . wp_fusion()->crm->name, array( 'source' => 'wpf-ecommerce' ) );

		return true;
	}

	/**
	 * Includes SureCart integration class.
	 *
	 * @since 1.25.0
	 */
	public function includes() {
		require_once WPF_EC_DIR_PATH . 'includes/integrations/surecart/class-surecart-integration.php';
	}

	/**
	 * Sends order data to CRM.
	 *
	 * @since 1.25.0
	 *
	 * @param SureCart\Models\Checkout $checkout The checkout.
	 */
	public function send_order_data( $checkout ) {

		$order_args = $this->get_order_args( $checkout );

		$contact_id = $this->get_contact_id( $order_args, $checkout );

		if ( is_wp_error( $contact_id ) ) {
			wpf_log( 'error', $order_args['user_id'], 'Error adding SureCart Order <a href="' . admin_url( 'admin.php?page=sc-orders&action=edit&id=' . $checkout->order ) . '" target="_blank">#' . $order_args['order_label'] . '</a>: unable to create contact record.', array( 'source' => 'wpf-surecart' ) );
			return;
		}

		// Add order.
		$result = wp_fusion_ecommerce()->crm->add_order( $checkout->order, $contact_id, $order_args );

		if ( is_wp_error( $result ) ) {

			wpf_log( 'error', $order_args['user_id'], 'Error adding SureCart Order <a href="' . admin_url( 'admin.php?page=sc-orders&action=edit&id=' . $order_id ) . '" target="_blank">#' . $order_args['order_label'] . '</a>: ' . $result->get_error_message(), array( 'source' => 'wpf-surecart' ) );
			return;
		}

		// Save invoice and complete date in checkout metadata.
		$params = array(
			'method' => 'PATCH',
			'body'   => wp_json_encode(
				array(
					'metadata' => array(
						'wpf_ec_' . wp_fusion()->crm->slug . '_invoice_id' => $result,
						'wpf_ec_complete' => current_time( 'Y-m-d H:i:s' ),
					),
				)
			),
		);

		$this->send_request( 'checkouts/' . $checkout->id, $params );

		do_action( 'wpf_surecart_complete', $checkout->order, $result, $contact_id, $order_args );
	}

	/**
	 * Gets the contact ID for the order.
	 *
	 * @since 1.25.1
	 *
	 * @param array $order_args The order args.
	 * @return int|WP_Error The contact ID or error.
	 */
	public function get_contact_id( $order_args, $checkout ) {

		$contact_id = false;

		// Check the user.
		if ( ! empty( $order_args['user_id'] ) ) {
			$contact_id = wp_fusion()->user->get_contact_id( $order_args['user_id'] );
		}

		// Check the CRM.
		if ( empty( $contact_id ) ) {
			$contact_id = wp_fusion()->crm->get_contact_id( $order_args['user_email'] );

			if ( is_wp_error( $contact_id ) ) {
				return $contact_id;
			}
		}

		if ( empty( $contact_id ) ) {

			$checkout_meta = $checkout->getAttributes();

			$contact_data = array(
				'user_email'  => $checkout_meta['customer']['email'],
				'first_name'  => isset( $checkout_meta['customer']['first_name'] ) ? $checkout_meta['customer']['first_name'] : '',
				'last_name'   => isset( $checkout_meta['customer']['last_name'] ) ? $checkout_meta['customer']['last_name'] : '',
				'line_1'      => isset( $checkout_meta['shipping_address']['line_1'] ) ? $checkout_meta['shipping_address']['line_1'] : '',
				'line_2'      => isset( $checkout_meta['shipping_address']['line_2'] ) ? $checkout_meta['shipping_address']['line_2'] : '',
				'city'        => isset( $checkout_meta['shipping_address']['city'] ) ? $checkout_meta['shipping_address']['city'] : '',
				'state'       => isset( $checkout_meta['shipping_address']['state'] ) ? $checkout_meta['shipping_address']['state'] : '',
				'country'     => isset( $checkout_meta['shipping_address']['country'] ) ? $checkout_meta['shipping_address']['country'] : '',
				'postal_code' => isset( $checkout_meta['shipping_address']['postal_code'] ) ? $checkout_meta['shipping_address']['postal_code'] : '',
			);

			// Custom Fields.
			foreach ( $checkout_meta['metadata'] as $key => $value ) {
				$contact_data[ $key ] = $value;
			}

			$contact_id = wp_fusion()->crm->add_contact( $contact_data );

			if ( is_wp_error( $contact_id ) ) {
				return $contact_id;
			}
		}

		return $contact_id;
	}

	/**
	 * Send renewed subscription data as a new order.
	 *
	 * @since 1.25.0
	 *
	 * @param SureCart\Models\Subscription $subscription The subscription.
	 */
	public function renew_subscription( $subscription ) {

		$purchase = Purchase::with(
			array(
				'initial_order',
				'order.checkout',
				'checkout.customer',
				'checkout.payment_intent',
				'checkout.line_items',
				'line_item.price',
				'price.product',
			)
		)->find( $subscription->purchase );

		$this->send_order_data( $purchase->initial_order->checkout );
	}


	/**
	 * Send request through SureCart API to update the CRM invoice ID.
	 *
	 * @since 1.25.0
	 *
	 * @param string $endpoint The endpoint.
	 * @param array  $params   The params.
	 *
	 * @return array|WP_Error The response.
	 */
	public function send_request( $endpoint, $params = array() ) {

		$api            = new SureCart\Models\ApiToken();
		$api_token      = $api->get();
		$default_params = array(
			'headers' => array(
				'authorization' => 'Bearer ' . $api_token,
				'content-type'  => 'application/json',
			),
		);

		$params = array_merge( $default_params, $params );

		$response = wp_safe_remote_request( 'https://api.surecart.com/v1/' . $endpoint, $params );

		return $response;
	}

	/**
	 * Formats the price for CRMs.
	 *
	 * @since 1.25.0
	 *
	 * @param string $price
	 *
	 * @return string The formatted price.
	 */
	public function format_price( $price ) {
		return number_format( (float) $price / 100, 2, '.', '' );
	}

	/**
	 * Get order arguments from checkout object.
	 *
	 * @since 1.25.0
	 *
	 * @param SureCart\Models\Checkout $checkout Checkout object.
	 * @return array The order arguments.
	 */
	public function get_order_args( $checkout ) {

		$products   = array();
		$line_items = array();

		foreach ( $checkout->line_items->data as $line_item ) {
			$products[] = $this->get_product_data( $line_item );
		}

		if ( $checkout->tax_amount > 0 ) {
			$line_items[] = array(
				'type'        => 'tax',
				'price'       => $this->format_price( $checkout->tax_amount ),
				'title'       => __( 'Tax', 'wp-fusion' ),
				'description' => __( 'Tax', 'wp-fusion' ),
			);
		}

		if ( $checkout->discount_amount < 0 ) {
			$line_items[] = array(
				'type'        => 'discount',
				'price'       => $this->format_price( $checkout->discount_amount ),
				'title'       => __( 'Discount', 'wp-fusion' ),
				'description' => sprintf( __( 'Discount code: %s', 'wp-fusion' ), $checkout->discount->promotion->code ),
				'code'        => $checkout->discount->promotion->code,
			);
		}

		if ( $checkout->shipping_amount > 0 ) {
			$line_items[] = array(
				'type'        => 'shipping',
				'price'       => $this->format_price( $checkout->shipping_amount ),
				'title'       => $checkout->selected_shipping_choice,
				'description' => $checkout->selected_shipping_choice,
			);
		}

		$user = get_user_by( 'email', $checkout->email );

		$currency_symbol = \SureCart\Support\Currency::getCurrencySymbol( strtoupper( $checkout->currency ) );
		$order_args      = array(
			'order_label'     => sprintf( __( 'SureCart Order #%s', 'wp-fusion' ), sanitize_text_field( $checkout->number ) ),
			'order_number'    => sanitize_text_field( $checkout->number ),
			'order_edit_link' => admin_url( 'admin.php?page=sc-orders&action=edit&id=' . $checkout->order ),
			'products'        => $products,
			'user_email'      => sanitize_email( $checkout->email ),
			'line_items'      => $line_items,
			'total'           => $this->format_price( $checkout->net_paid_amount ),
			'currency'        => strtoupper( sanitize_text_field( $checkout->currency ) ),
			'currency_symbol' => sanitize_text_field( $currency_symbol ),
			'order_date'      => gmdate( 'U', $checkout->created_at ),
			'provider'        => 'surecart',
			'status'          => sanitize_text_field( $checkout->status ),
			'user_id'         => ! empty( $user ) ? $user->ID : 0,
		);

		// Important to update instead of create in batch operations.
		if ( isset( $checkout->metadata->{ 'wpf_ec_' . wp_fusion()->crm->slug . '_invoice_id' } ) ) {
			$order_args['invoice_id'] = $checkout->metadata->{ 'wpf_ec_' . wp_fusion()->crm->slug . '_invoice_id' };
		}

		// Sometimes the payment intent is not available while doing batch operations for some orders.
		if ( isset( $checkout->payment_intent->processor_type ) ) {
			$order_args['payment_method'] = sanitize_text_field( $checkout->payment_intent->processor_type );
		} else {
			$payment_intent               = PaymentIntent::find( $checkout->payment_intent );
			$order_args['payment_method'] = sanitize_text_field( $payment_intent->processor_type );
		}

		$order_args = apply_filters( 'wpf_ecommerce_order_args', $order_args, $checkout->id );

		return $order_args;
	}

	/**
	 * Gets the product data to use in order args.
	 *
	 * @since 1.25.0
	 *
	 * @param SureCart\Models\LineItem $line_item The line item.
	 * @return array The product data.
	 */
	public function get_product_data( $line_item ) {

		$product = $line_item->price->product;

		$product_data = array(
			'id'             => sanitize_text_field( $product->id ),
			'name'           => sanitize_text_field( $product->name ) . ' - ' . sanitize_text_field( $line_item->price->name ),
			'crm_product_id' => $this->get_crm_product_id( $line_item->price ),
			'sku'            => sanitize_text_field( $product->sku ),
			'image'          => esc_url( $product->image_url ),
			'price'          => $this->format_price( $line_item->price->full_amount ),
			'stock_quantity' => intval( $product->available_stock ),
			'qty'            => intval( $line_item->quantity ),
		);

		if ( $line_item->variant ) {
			$product_data['id']        = sanitize_text_field( $line_item->variant->id );
			$product_data['parent_id'] = sanitize_text_field( $product->id );

			if ( ! empty( $line_item->variant->amount ) ) {
				$product_data['price'] = $this->format_price( $line_item->variant->amount );
			}

			if ( ! empty( $line_item->variant->sku ) ) {
				$product_data['sku'] = sanitize_text_field( $line_item->variant->sku );
			}

			if ( ! empty( $line_item->variant->image_url ) ) {
				$product_data['image'] = esc_url( $line_item->variant->image_url );
			}

			if ( ! empty( $line_item->variant->available_stock ) ) {
				$product_data['stock_quantity'] = intval( $line_item->variant->available_stock );
			}
		}

		return $product_data;
	}

	/**
	 * Get the CRM product ID for a product.
	 *
	 * @since 1.25.0
	 *
	 * @param SureCart\Models\Price $price The price.
	 * @return bool|int The CRM product ID or false if not found.
	 */
	public function get_crm_product_id( $price ) {

		$product = $price->product;

		$crm_product_id = false;

		$integrations = Integration::where( 'model_id', $product->id )->andWhere( 'provider', 'wp-fusion/ecommerce' )->get();

		foreach ( $integrations as $integration ) {

			// Make sure it's the correct price ID.
			if ( $integration->price_id && $price->id && $integration->price_id !== $price->id ) {
				continue;
			}

			$crm_product_id = $integration->integration_id;
		}

		// See if we can find a match based on the product name.
		$available_products = get_option( 'wpf_' . wp_fusion()->crm->slug . '_products', array() );

		// Check for deleted / invalid products.
		if ( ! empty( $crm_product_id ) && ! array_key_exists( $crm_product_id, $available_products ) ) {
			$crm_product_id = false;
		}

		// If it's not set try and find a match based on the product name.
		if ( empty( $crm_product_id ) && is_array( $available_products ) ) {
			$name           = $product->name . ' - ' . $price->name;
			$crm_product_id = array_search( $name, $available_products );
		}

		return $crm_product_id;
	}


	/**
	 * //
	 * // BATCH TOOLS
	 * //
	 **/

	/**
	 * Adds SureCart checkbox to available export options
	 *
	 * @since 1.25.0
	 *
	 * @param array $options The export options.
	 * @return array The export options.
	 */
	public function export_options( $options ) {

		$options['surecart_ecom'] = array(
			'label'         => __( 'SureCart orders (Ecommerce addon)', 'wp-fusion' ),
			'title'         => 'Orders',
			'process_again' => true,
			'tooltip'       => sprintf( __( 'Syncs your SureCart orders to %s. Does not apply any tags or update any contact fields.', 'wp-fusion' ), wp_fusion()->crm->name ),
		);

		return $options;
	}

	/**
	 * Gets the orders to be exported.
	 *
	 * @since  1.25.0
	 *
	 * @param  array $args   The search arguments.
	 * @return array|WP_Error The order IDs or error.
	 */
	public function batch_init( $args ) {

		$checkout_ids = array();
		$page         = 1;
		$continue     = true;

		$params = array( 'method' => 'GET' );

		while ( $continue ) {

			$checkouts = $this->send_request( '/checkouts?limit=100&page=' . $page, $params );

			if ( is_wp_error( $checkouts ) ) {
				return $checkouts;
			}

			$checkouts = json_decode( wp_remote_retrieve_body( $checkouts ) );

			if ( ! $checkouts || empty( $checkouts->data ) ) {
				break;
			}

			if ( ! empty( $args['skip_processed'] ) ) {
				foreach ( $checkouts->data as $checkout ) {
					if ( isset( $checkout->metadata ) && ! empty( $checkout->metadata->{ 'wpf_ec_complete' } ) ) {
						continue; // Already processed.
					}
				}
			}

			$checkout_ids = array_merge( $checkout_ids, wp_list_pluck( $checkouts->data, 'id' ) );

			if ( count( $checkouts->data ) < 100 ) {
				$continue = false;
			} else {
				++$page;
			}
		}

		return $checkout_ids;
	}


	/**
	 * Processes order actions in steps.
	 *
	 * @since 1.25.0
	 *
	 * @param string $order_id The order ID.
	 */
	public function batch_step( $order_id ) {

		$checkout = Checkout::with(
			array(
				'customer',
				'payment_intent',
				'line_items',
				'line_item.price',
				'price.product',
			)
		)->find( $order_id );

		$this->send_order_data( $checkout );
	}
}

new WPF_EC_SureCart();
