<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Abandoned Cart integration.
 *
 * @since 1.6.0
 */
class WPF_ET_Abandoned_Cart extends WPF_ET_Integrations_Base {

	/**
	 * The slug for WP Fusion's module tracking.
	 *
	 * @since 1.6.0
	 * @var string $slug
	 */
	public $slug = 'abandoned-cart';

	/**
	 * The plugin name for WP Fusion's module tracking.
	 *
	 * @since 1.6.0
	 * @var string $name
	 */
	public $name = 'Abandoned Carts';

	/**
	 * Get things started
	 *
	 * @access public
	 * @return void
	 */
	public function init() {

		add_action( 'wpf_abandoned_cart_created', array( $this, 'cart_created' ), 10, 2 );
	}

	/**
	 * Gets the triggers for the integration.
	 *
	 * @access protected
	 *
	 * @since  1.6.0
	 *
	 * @return array The triggers.
	 */
	protected function setup_triggers() {

		$triggers = array(
			'cart_created' => array(
				'name'         => __( 'Abandoned Cart Created', 'wp-fusion-event-tracking' ),
				'description'  => __( 'Triggered each time an abandoned cart has been created.', 'wp-fusion-event-tracking' ),
				'has_global'   => true,
				'option_types' => array( 'cart' ),
			),
		);

		return $triggers;
	}


	/**
	 * Gets the cart options.
	 *
	 * @since  1.6.0
	 *
	 * @return array The cart options.
	 */
	public function get_cart_options() {

		return array(
			'name'    => __( 'Cart', 'wp-fus ion-event-tracking' ),
			'type'    => 'cart',
			'options' => array(
				array(
					'meta'        => 'cart_id',
					'preview'     => 32,
					'placeholder' => __( 'The cart ID', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'recovery_url',
					'preview'     => home_url( '/checkout/?wpfrc=123' ),
					'placeholder' => __( 'The cart recovery URL', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'source',
					'preview'     => 'woocommerce',
					'placeholder' => __( 'The cart source', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'contact_id',
					'preview'     => 3,
					'placeholder' => __( 'The cart contact id', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'first_name',
					'preview'     => 'John',
					'placeholder' => __( 'The user first name', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'last_name',
					'preview'     => 'Doe',
					'placeholder' => __( 'The user last name', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'user_email',
					'preview'     => 'example@email.com',
					'placeholder' => __( 'The user email address', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'cart_value',
					'preview'     => 72,
					'placeholder' => __( 'the cart total value', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'currency',
					'preview'     => 'USD',
					'placeholder' => __( 'the cart currency', 'wp-fusion-event-tracking' ),
				),
			),
		);
	}


	/**
	 * Gets the cart variables.
	 *
	 * @since  1.6.0
	 *
	 * @param  array $args The cart arguments.
	 * @return array The cart variables.
	 */
	public function get_cart_vars( $args ) {
		$args['first_name'] = $args['user_data']['first_name'];
		$args['last_name']  = $args['user_data']['last_name'];
		$args['cart_value'] = $args['update_data']['cart_value'];

		return array(
			'cart' => $args,
		);
	}


	/**
	 * Fires when an abandoned cart is created.
	 *
	 * @param string|integer $contact_id
	 * @param array          $cart_args
	 */
	public function cart_created( $contact_id, $cart_args ) {
		$email_address = $cart_args['user_email'];

		$events = $this->get_events( 'cart_created' );

		if ( ! empty( $events ) ) {

			$args = $this->get_cart_vars( $cart_args );

			foreach ( $events as $event ) {
				$event = $this->replace_tags( $event, $args );
				$this->track_event( $event, $email_address );
			}
		}
	}
}

new WPF_ET_Abandoned_Cart();
