<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Affiliate WP integration.
 *
 * @since 1.6.0
 */
class WPF_ET_AffiliateWP extends WPF_ET_Integrations_Base {

	/**
	 * The slug for WP Fusion's module tracking.
	 *
	 * @since 1.6.0
	 * @var string $slug
	 */

	public $slug = 'affiliate-wp';

	/**
	 * The plugin name for WP Fusion's module tracking.
	 *
	 * @since 1.6.0
	 * @var string $name
	 */
	public $name = 'Affiliate WP';

	/**
	 * Get things started.
	 *
	 * @since 1.6.0
	 */
	public function init() {

		add_action( 'affwp_insert_affiliate', array( $this, 'add_affiliate' ), 15 );

		add_action( 'affwp_set_affiliate_status', array( $this, 'affiliate_status_updated' ), 10, 2 );

		add_action( 'affwp_post_insert_visit', array( $this, 'add_visit' ), 10, 3 );

		add_action( 'affwp_referral_accepted', array( $this, 'add_referral' ), 10, 2 );

	}




	/**
	 * Gets the triggers for the integration.
	 *
	 * @access protected
	 *
	 * @since  1.6.0
	 *
	 * @return array The triggers.
	 */
	protected function setup_triggers() {

		$triggers = array(
			'affiliate_created' => array(
				'name'         => __( 'New Affiliate', 'wp-fusion-event-tracking' ),
				'description'  => __( 'Triggered each time an affiliate registers.', 'wp-fusion-event-tracking' ),
				'has_global'   => true,
				'option_types' => array( 'affiliate' ),
			),
			'affiliate_status_updated' => array(
				'name'         => __( 'Affiliate Status Updated', 'wp-fusion-event-tracking' ),
				'description'  => __( 'Triggered each time an affiliate\'s status is updated.', 'wp-fusion-event-tracking' ),
				'has_global'   => true,
				'option_types' => array( 'affiliate' ),
			),
			'link_visited'      => array(
				'name'         => __( 'New Visit', 'wp-fusion-event-tracking' ),
				'description'  => __( 'Triggered each time a referral visits a link.' ),
				'has_global'   => true,
				'option_types' => array( 'affiliate', 'visit' ),
			),
			'referral_earned'   => array(
				'name'         => __( 'Referral Earned', 'wp-fusion-event-tracking' ),
				'description'  => __( 'Triggered each time a referral is earned.' ),
				'has_global'   => true,
				'option_types' => array( 'affiliate', 'visit', 'referral' ),
			),
		);

		return $triggers;
	}



	/**
	 * Gets the affiliate options.
	 *
	 * @since  1.6.0
	 *
	 * @return array The order options.
	 */
	public function get_affiliate_options() {
		return array(
			'name'    => __( 'Affiliate', 'wp-fusion-event-tracking' ),
			'type'    => 'affiliate',
			'options' => array(
				array(
					'meta'        => 'affiliate_id',
					'preview'     => 33,
					'placeholder' => __( 'The affiliate ID', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'payment_email',
					'preview'     => 'affiliate@paypal.com',
					'placeholder' => __( 'The affiliate paypal email', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'rate',
					'preview'     => '10',
					'placeholder' => __( 'The referral rate', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'earnings',
					'preview'     => '500',
					'placeholder' => __( 'The total earnings', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'referrals',
					'preview'     => '21',
					'placeholder' => __( 'The total number of referrals', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'visits',
					'preview'     => '55',
					'placeholder' => __( 'The total number of visits', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'status',
					'preview'     => 'Active',
					'placeholder' => __( 'The affiliate status', 'wp-fusion-event-tracking' ),
				),

			),
		);
	}

	/**
	 * Gets the affiliate variables.
	 *
	 * @since  1.6.0
	 *
	 * @param  int $affiliate_id The affiliate ID.
	 * @return array The affiliate variables.
	 */
	public function get_affiliate_vars( $affiliate_id ) {

		$affiliate = affwp_get_affiliate( $affiliate_id );
		if ( empty( $affiliate ) ) {
			return array();
		}

		$affiliate_fields = array();

		$meta_column = array_column( $this->get_affiliate_options()['options'], 'meta' );

		if ( $affiliate->rate == '' ) {
			$affiliate->rate = affiliate_wp()->settings->get( 'referral_rate', 20 );
		}

		// Post/meta fields.
		foreach ( $meta_column as $meta_key ) {
			if ( isset( $affiliate->$meta_key ) && $affiliate->$meta_key != '' ) {
				$affiliate_fields[ $meta_key ] = $affiliate->$meta_key;
			}
		}

		return array(
			'affiliate' => $affiliate_fields,
		);
	}



	/**
	 * Fires when an affiliate has been added or registered.
	 *
	 * @since  1.6.0
	 *
	 * @param integer $affiliate_id
	 */
	public function add_affiliate( $affiliate_id ) {

		$events = $this->get_events( 'affiliate_created' );

		if ( ! empty( $events ) ) {

			$affiliate = affwp_get_affiliate( $affiliate_id );
			$user      = get_user_by( 'id', $affiliate->user_id );

			$args = $this->get_affiliate_vars( $affiliate_id );

			foreach ( $events as $event ) {
				$event = $this->replace_tags( $event, $args );
				$this->track_event( $event, $user->user_email );
			}
		}
	}

	/**
	 * Fires when an affiliate's status is updated.
	 *
	 * @since  1.6.0
	 *
	 * @param int    $affiliate_id The affiliate ID.
	 * @param string $status       The affiliate status.
	 */
	public function affiliate_status_updated( $affiliate_id = 0, $status = '' ) {

		$events = $this->get_events( 'affiliate_status_updated' );

		if ( ! empty( $events ) ) {

			$affiliate = affwp_get_affiliate( $affiliate_id );
			$user      = get_user_by( 'id', $affiliate->user_id );

			$args = $this->get_affiliate_vars( $affiliate_id );

			$args['affiliate']['status'] = ucwords( $status );

			foreach ( $events as $event ) {
				$event = $this->replace_tags( $event, $args );
				$this->track_event( $event, $user->user_email );
			}
		}
	}

	/**
	 * Gets the link visits options.
	 *
	 * @since  1.6.0
	 *
	 * @return array the link visits options.
	 */
	public function get_visit_options() {

		return array(
			'name'    => __( 'Link Visit', 'wp-fusion-event-tracking' ),
			'type'    => 'visit',
			'options' => array(
				array(
					'meta'        => 'visit_id',
					'preview'     => 33,
					'placeholder' => __( 'The visit ID', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'url',
					'preview'     => home_url( '/example-page/' ),
					'placeholder' => __( 'The visited page', 'wp-fusion-event-tracking' ),
				),
			),
		);
	}

	/**
	 * Gets the visit variables.
	 *
	 * @since  1.6.0
	 *
	 * @param  int $visit_id The visit ID.
	 * @return array The visit variables.
	 */
	public function get_visit_vars( $visit_id ) {

		$visit = affwp_get_visit( $visit_id );

		if ( empty( $visit ) ) {
			return array();
		}

		$visits_fields = array();

		$meta_column = array_column( $this->get_visit_options()['options'], 'meta' );

		// Post/meta fields.
		foreach ( $meta_column as $meta_key ) {
			if ( isset( $visit->$meta_key ) && $visit->$meta_key != '' ) {
				$visits_fields[ $meta_key ] = $visit->$meta_key;
			}
		}

		return array(
			'visit' => $visits_fields,
		);

	}

	/**
	 * Fires when a referral visits a link.
	 *
	 * @since  1.6.0
	 *
	 * @param integer $visit_id
	 * @param array   $data
	 */
	public function add_visit( $visit_id, $data ) {

		$events = $this->get_events( 'link_visited' );

		if ( ! empty( $events ) ) {

			$affiliate = affwp_get_affiliate( $data['affiliate_id'] );
			$user      = get_user_by( 'id', $affiliate->user_id );

			$args = array_merge(
				$this->get_affiliate_vars( $data['affiliate_id'] ),
				$this->get_visit_vars( $visit_id ),
				wp_fusion_event_tracking()->integration( 'user' )->get_user_vars( $user->ID ),
			);

			foreach ( $events as $event ) {
				$event = $this->replace_tags( $event, $args );
				$this->track_event( $event, $user->user_email );
			}
		}

	}


	/**
	 * Gets the referral options.
	 *
	 * @since  1.6.0
	 *
	 * @return array the referral options.
	 */
	public function get_referral_options() {

		return array(
			'name'    => __( 'Referral', 'wp-fusion-event-tracking' ),
			'type'    => 'referral',
			'options' => array(
				array(
					'meta'        => 'referral_id',
					'preview'     => 33,
					'placeholder' => __( 'The referral ID', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'status',
					'preview'     => 'Pending',
					'placeholder' => __( 'The referral status', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'amount',
					'preview'     => '50',
					'placeholder' => __( 'The referral earned amount', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'currency',
					'preview'     => 'USD',
					'placeholder' => __( 'The referral currency', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'context',
					'preview'     => 'woocommerce',
					'placeholder' => __( 'The referral context', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'reference',
					'preview'     => '5',
					'placeholder' => __( 'The referral reference', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'customer_id',
					'preview'     => '3',
					'placeholder' => __( 'The referral customer id', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'customer_email',
					'preview'     => 'customer@email.com',
					'placeholder' => __( 'The referral customer email', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'type',
					'preview'     => 'sale',
					'placeholder' => __( 'The referral type (Sale, Lead, Optin)', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'date',
					'preview'     => '2024-07-24 22:36:31',
					'placeholder' => __( 'The referral date', 'wp-fusion-event-tracking' ),
				),
			),
		);
	}

	/**
	 * Gets the referral variables.
	 *
	 * @since  1.6.0
	 *
	 * @param  int $referral_id The referral ID.
	 * @return array The referral variables.
	 */
	public function get_referral_vars( $referral_id ) {

		$referral = affwp_get_referral( $referral_id );

		if ( empty( $referral ) ) {
			return array();
		}

		$referrals_fields = array();

		$meta_column = array_column( $this->get_referral_options()['options'], 'meta' );

		// Add customer email.
		if ( $referral->customer_id != '' ) {
			$customer = get_user_by( 'id', $referral->customer_id );
			if ( $customer ) {
				$referral->customer_email = $customer->user_email;
			}
		}

		foreach ( $meta_column as $meta_key ) {
			if ( isset( $referral->$meta_key ) && $referral->$meta_key != '' ) {
				$referrals_fields[ $meta_key ] = $referral->$meta_key;
			}
		}

		return array(
			'referral' => $referrals_fields,
		);

	}

	/**
	 * Fires when a referral has been accepted.
	 *
	 * @since  1.6.0
	 *
	 * @param integer $affiliate_id
	 * @param object  $referral
	 */
	public function add_referral( $affiliate_id, $referral ) {
		$events = $this->get_events( 'referral_earned' );

		if ( ! empty( $events ) ) {

			$affiliate = affwp_get_affiliate( $affiliate_id );
			$user      = get_user_by( 'id', $affiliate->user_id );

			$args = array_merge(
				$this->get_affiliate_vars( $affiliate_id ),
				$this->get_visit_vars( $referral->visit_id ),
				$this->get_referral_vars( $referral->referral_id ),
				wp_fusion_event_tracking()->integration( 'user' )->get_user_vars( $user->ID ),
			);

			foreach ( $events as $event ) {
				$event = $this->replace_tags( $event, $args );
				$this->track_event( $event, $user->user_email );
			}
		}
	}

}

new WPF_ET_AffiliateWP();
