<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Give WP integration.
 *
 * @since 1.6.0
 */
class WPF_ET_Give extends WPF_ET_Integrations_Base {

	/**
	 * The slug for WP Fusion's module tracking.
	 *
	 * @since 1.6.0
	 * @var string $slug
	 */

	public $slug = 'give';

	/**
	 * The plugin name for WP Fusion's module tracking.
	 *
	 * @since 1.6.0
	 * @var string $name
	 */
	public $name = 'Give';

	/**
	 * Get things started.
	 *
	 * @since 1.6.0
	 */
	public function init() {

		add_action( 'give_insert_payment', array( $this, 'new_donation' ), 10, 2 );
	}


	/**
	 * Gets the triggers for the integration.
	 *
	 * @access protected
	 *
	 * @since  1.6.0
	 *
	 * @return array The triggers.
	 */
	protected function setup_triggers() {

		$triggers = array(
			'new_donation' => array(
				'name'         => __( 'New Donation', 'wp-fusion-event-tracking' ),
				'description'  => __( 'Triggered each time a donation has been made.', 'wp-fusion-event-tracking' ),
				'has_global'   => true,
				'option_types' => array( 'donation' ),
			),
		);

		return $triggers;
	}



	/**
	 * Gets the donation options.
	 *
	 * @since  1.6.0
	 *
	 * @return array The donation options.
	 */
	public function get_donation_options() {
		return array(
			'name'    => __( 'Donation', 'wp-fusion-event-tracking' ),
			'type'    => 'donation',
			'options' => array(
				array(
					'meta'        => 'ID',
					'preview'     => 33,
					'placeholder' => __( 'The donation ID', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'first_name',
					'preview'     => 'Jon',
					'placeholder' => __( 'The donor first name', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'last_name',
					'preview'     => 'Doe',
					'placeholder' => __( 'The donor last name', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'email',
					'preview'     => 'jon@email.com',
					'placeholder' => __( 'The donor email address', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'country',
					'preview'     => 'United States',
					'placeholder' => __( 'The donor country', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'city',
					'preview'     => 'New York',
					'placeholder' => __( 'The donor city', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'subtotal',
					'preview'     => '100',
					'placeholder' => __( 'The donation amount', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'currency',
					'preview'     => 'USD',
					'placeholder' => __( 'The donation currency', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'status',
					'preview'     => 'pending',
					'placeholder' => __( 'The donation payment status', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'payment_gateway',
					'preview'     => 'Paypal',
					'placeholder' => __( 'The payment gateway', 'wp-fusion-event-tracking' ),
				),
				array(
					'meta'        => 'form_title',
					'preview'     => 'Donation Form',
					'placeholder' => __( 'The donation form title', 'wp-fusion-event-tracking' ),
				),

			),
		);
	}

	/**
	 * Gets the donation variables.
	 *
	 * @since  1.6.0
	 *
	 * @param  int $donation_id The donation ID.
	 * @return array The donation variables.
	 */
	public function get_donation_vars( $donation_id ) {
		$donation = new Give_Payment( $donation_id );

		if ( empty( $donation ) ) {
			return array();
		}

		$donation_fields = array();

		$meta_column = array_column( $this->get_donation_options()['options'], 'meta' );

		$payment_meta              = $donation->payment_meta;
		$donation->payment_gateway = $payment_meta['_give_payment_gateway'];
		$donation->country         = $payment_meta['_give_donor_billing_country'];
		$donation->city            = $payment_meta['_give_donor_billing_city'];
		$donation->form_title      = $payment_meta['_give_payment_form_title'];

		// Post/meta fields.
		foreach ( $meta_column as $meta_key ) {
			if ( isset( $donation->$meta_key ) && $donation->$meta_key != '' ) {
				$donation_fields[ $meta_key ] = $donation->$meta_key;
			}
		}

		return array(
			'donation' => $donation_fields,
		);
	}



	/**
	 * Fires when a donation payment has been made.
	 *
	 * @since  1.6.0
	 *
	 * @param integer $donation_id
	 */
	public function new_donation( $donation_id ) {

		$events = $this->get_events( 'new_donation' );

		if ( ! empty( $events ) ) {

			$donation = new Give_Payment( $donation_id );
			if ( $user->user_id ) {
				$user          = get_user_by( 'id', $user->user_id );
				$email_address = $user->user_email;
			} else {
				$email_address = $donation->email;
			}

			$args = $this->get_donation_vars( $donation_id );

			foreach ( $events as $event ) {
				$event = $this->replace_tags( $event, $args );
				$this->track_event( $event, $email_address );
			}
		}
	}

}

new WPF_ET_Give();
