<?php

/*
Plugin Name: WP Fusion - Logins Addon
Description: Tools for WordPress user logins, including applying tags on first login, tracking stale accounts, and redirecting on login based on tags.
Plugin URI: https://wpfusionplugin.com/
Version: 1.4.0
Author: Very Good Plugins
Author URI: https://verygoodplugins.com/
Text Domain: wp-fusion-logins
*/

/**
 * @copyright Copyright (c) 2016. All rights reserved.
 *
 * @license   Released under the GPL license http://www.opensource.org/licenses/gpl-license.php
 *
 * **********************************************************************
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * **********************************************************************
 */

define( 'WPF_LOGINS_VERSION', '1.4.0' );

// deny direct access
if ( ! function_exists( 'add_action' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit();
}


final class WP_Fusion_Logins {

	/** Singleton *************************************************************/

	/**
	 * @var WP_Fusion_Logins The one true WP_Fusion_Logins
	 * @since 1.0
	 */
	private static $instance;

	/**
	 * Allows interfacing with the main class
	 *
	 * @var WPF_Logins_Public
	 * @since 2.0
	 */
	public $public;

	/**
	 * Allows interfacing with the compat class
	 *
	 * @var WPF_Logins_Compat
	 * @since 2.0
	 */
	public $compat;


	/**
	 * Main WP_Fusion_Logins Instance
	 *
	 * Insures that only one instance of WP_Fusion_Logins exists in memory at any one
	 * time. Also prevents needing to define globals all over the place.
	 *
	 * @since 1.0
	 * @static
	 * @staticvar array $instance
	 * @return The one true WP_Fusion_Logins
	 */

	public static function instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof WP_Fusion_Logins ) ) {

			self::$instance = new WP_Fusion_Logins();
			self::$instance->setup_constants();
			self::$instance->includes();

			if ( wp_fusion()->settings->get( 'connection_configured' ) ) {

				self::$instance->public = new WPF_Logins_Public();
				self::$instance->compat = new WPF_Logins_Compat();

			}

			self::$instance->updater();

			add_action( 'plugins_loaded', array( self::$instance, 'load_textdomain' ) );

		}

		return self::$instance;
	}

	/**
	 * Throw error on object clone
	 *
	 * The whole idea of the singleton design pattern is that there is a single
	 * object therefore, we don't want the object to be cloned.
	 *
	 * @access protected
	 * @return void
	 */

	public function __clone() {
		// Cloning instances of the class is forbidden
		_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?', 'wp-fusion-logins' ), '1.6' );
	}

	/**
	 * Disable unserializing of the class
	 *
	 * @access protected
	 * @return void
	 */

	public function __wakeup() {
		// Unserializing instances of the class is forbidden
		_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?', 'wp-fusion-logins' ), '1.6' );
	}

	/**
	 * Setup plugin constants
	 *
	 * @access private
	 * @return void
	 */

	private function setup_constants() {

		if ( ! defined( 'WPF_LOGINS_DIR_PATH' ) ) {
			define( 'WPF_LOGINS_DIR_PATH', plugin_dir_path( __FILE__ ) );
		}

		if ( ! defined( 'WPF_LOGINS_PLUGIN_PATH' ) ) {
			define( 'WPF_LOGINS_PLUGIN_PATH', plugin_basename( __FILE__ ) );
		}

		if ( ! defined( 'WPF_LOGINS_DIR_URL' ) ) {
			define( 'WPF_LOGINS_DIR_URL', plugin_dir_url( __FILE__ ) );
		}

	}


	/**
	 * Include required files
	 *
	 * @access private
	 * @return void
	 */

	private function includes() {

		if ( is_admin() ) {
			require_once WPF_LOGINS_DIR_PATH . 'includes/admin/class-admin.php';
		}

		require_once WPF_LOGINS_DIR_PATH . 'includes/class-public.php';
		require_once WPF_LOGINS_DIR_PATH . 'includes/class-compat.php';

	}

	/**
	 * Load internationalization files
	 *
	 * @access public
	 * @return void
	 */

	public function load_textdomain() {

		load_plugin_textdomain( 'wp-fusion-logins', false, 'wp-fusion-logins/languages' );

	}

	/**
	 * Set up EDD updater
	 *
	 * @access public
	 * @return void
	 */

	public function updater() {

		// To support auto-updates, this needs to run during the wp_version_check cron job for privileged users.
		$doing_cron = defined( 'DOING_CRON' ) && DOING_CRON;

		if ( ! current_user_can( 'manage_options' ) && ! $doing_cron ) {
			return;
		}

		if ( class_exists( 'WPF_Plugin_Updater' ) ) {

			// setup the updater
			new WPF_Plugin_Updater(
				WPF_STORE_URL, __FILE__, array(
					'version' => WPF_LOGINS_VERSION,
					'license' => wpf_get_option( 'license_key' ),
					'item_id' => 9368,
					'author'  => 'Very Good Plugins',
				)
			);
		}
	}
}


/**
 * The main function responsible for returning the one true WP Fusion Logins
 * Instance to functions everywhere.
 *
 * Use this function like you would a global variable, except without needing
 * to declare the global.
 *
 * Example: <?php $wpf_vt = wp_fusion_logins(); ?>
 *
 * @return object The one true WP Fusion Logins
 */

function wp_fusion_logins() {

	if ( ! function_exists( 'wp_fusion' ) ) {
		return;
	}

	return WP_Fusion_Logins::instance();
}

add_action( 'wp_fusion_init', 'wp_fusion_logins' );