import assign from 'lodash.assign';
import TagRepeater from "./components/tag-repeater";
import { useState, useEffect } from '@wordpress/element';
import apiFetch from '@wordpress/api-fetch';
import WpfSelect from "@verygoodplugins/wpfselect";

const { addFilter } = wp.hooks;
const { __ } = wp.i18n;
const { createHigherOrderComponent } = wp.compose;
const { Fragment } = wp.element;
const { InspectorControls } = wp.blockEditor;
const { PanelBody, BaseControl } = wp.components;

// Enable spacing control on the following blocks
const enablePrestoPlayerBlocks = [
	'presto-player/self-hosted',
	'presto-player/youtube',
	'presto-player/bunny',
	'presto-player/vimeo',
	'core/video',
	'core/embed'
];

/**
 * Add media tools attributes to block.
 *
 * @param {object} settings Current block settings.
 * @param {string} name Name of block.
 *
 * @returns {object} Modified block settings.
 */
const addMediaToolsAttributes = ( settings, name ) => {
	// Do nothing if it's another block than our defined ones.
	if ( ! enablePrestoPlayerBlocks.includes( name ) ) {
		return settings;
	}

	// Use Lodash's assign to gracefully handle if attributes are undefined
	settings.attributes = assign( settings.attributes, {
		wpf_tags_start: {
			type: 'array'
		},
		wpf_tags_complete: {
			type: 'array'
		},
		wpf_timecode_tags: {
			type: 'array'
		}
	} );

	return settings;
};

addFilter( 'blocks.registerBlockType', 'extend-presto-player/wpfusion/media-tools', addMediaToolsAttributes );

/*
* Add WP Fusion tag options to inspector controls of block.
*/
const withWPFusionControl = createHigherOrderComponent( ( BlockEdit ) => {
	return ( props ) => {

		// Do nothing if it's another block than our defined ones.
		if ( ! enablePrestoPlayerBlocks.includes( props.name ) ) {
			return (
				<BlockEdit { ...props } />
			);
		}

		const { wpf_tags_start, wpf_tags_complete  } = props.attributes;

		const [ availableTags, setAvailableTags ] = useState( null );

		useEffect( () => {
			apiFetch( { path: '/wp-fusion/secure-blocks/v1/available-tags/' } ).then(
				( result ) => {
					setAvailableTags( result );
				} );
		}, [] );

		return (
			<Fragment>
				<BlockEdit { ...props } />
				<InspectorControls>
					<PanelBody
						title={ __( 'WP Fusion', 'wp-fusion-media-tools' ) }
						initialOpen={ true }
					>
						<BaseControl
							label = { __( 'Start Tags', 'wp-fusion-media-tools' ) }
						>
							<WpfSelect
								existingTags = { wpf_tags_start }
								onChange = {
									( startTag ) => {
										props.setAttributes( {
											wpf_tags_start: startTag
										} );
									}
								}
								elementID = 'wpf-start-tags-select'
								{ ...props }
							/>
						</BaseControl>

						<TagRepeater { ...props } />

						<BaseControl
							label = { __( 'End Tags', 'wp-fusion-media-tools' ) }
						>
							<WpfSelect
								existingTags = { wpf_tags_complete  }
								onChange = {
									( completeTags ) => {
										props.setAttributes( {
											wpf_tags_complete: completeTags
										} );
									}
								}
								elementID = 'wpf-complete-tags-select'
								{ ...props }
							/>
						</BaseControl>
					</PanelBody>
				</InspectorControls>
			</Fragment>
		);
	};
}, 'withWPFusionControl' );

addFilter( 'editor.BlockEdit', 'extend-presto-player/wpfusion/media-tools/with-controls', withWPFusionControl );
